/*
 * Copyright 2015 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package ru.ispras.retrascope.engine.basis;

import ru.ispras.retrascope.basis.Engine;
import ru.ispras.retrascope.basis.EntityType;
import ru.ispras.retrascope.result.test.SequenceModel;

/**
 * The abstract class for engines of the simulator type.
 *
 * @author <a href="mailto:i.melnichenko@deltasolutions.ru">Igor Melnichenko</a>
 *
 * @see Engine
 * @see SequenceModel
 */
public abstract class SimulatorEngine extends Engine {
  /**
   * Creates an engine with the specified ID and type of a simulated entity as input type.
   * {@link SequenceModel#TYPE} is also registered as both input and output types.
   *
   * @param engineId the ID of the specified engine
   * @param simulatedEntity the type of simulated entity
   */
  protected SimulatorEngine(final String engineId, final EntityType simulatedEntity) {
    super(engineId, SequenceModel.TYPE, SequenceModel.TYPE);
    super.addInputType(simulatedEntity);
  }
}
