/*
 * Copyright 2018 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package ru.ispras.retrascope.engine.cfg.printer.c;

import org.stringtemplate.v4.ST;
import ru.ispras.castle.codegen.FileGenerator;
import ru.ispras.castle.codegen.FileGeneratorStringTemplate;
import ru.ispras.fortress.expression.NodeVariable;
import ru.ispras.fortress.util.Pair;
import ru.ispras.retrascope.basis.exception.RetrascopeException;
import ru.ispras.retrascope.model.basis.Module;
import ru.ispras.retrascope.model.cfg.CfgEmptyVisitor;

import java.io.IOException;
import java.util.Date;
import java.util.LinkedList;
import java.util.List;

/**
 * Control flow graph visitor for C language printer.
 *
 * @author <a href="mailto:smolov@ispras.ru">Sergey Smolov</a>
 */
public class CfgCVisitor extends CfgEmptyVisitor {

  private String outFileName;
  private String templateFile;
  private List<Pair<String, Object>> templateData;

  /**
   * Default constructor.
   */
  CfgCVisitor() {

    /* Empty. */
  }

  private void addData(final String key, final Object value) {

    this.templateData.add(new Pair<>(key, value));
  }

  /**
   * Initialize the object with the target file name.
   * @param outFileName The name for file to be generated by the object.
   * @param template The path to template file.
   */
  public void initialize(final String outFileName, final String template) {

    this.outFileName = outFileName;
    this.templateFile = template;
  }

  @Override
  public void onRootBegin() {
    this.templateData = new LinkedList<>();
    addData("time", new Date().toString());
  }

  @Override
  public void onRootEnd() {
    final FileGenerator generator =
        new FileGeneratorStringTemplate(this.outFileName, new String[] {this.templateFile},
            stGroup -> {
              final ST st = stGroup.getInstanceOf("cfg_model");

              for (Pair<String, Object> entry : this.templateData) {
                st.add(entry.first, entry.second);
              }
              return st;
            });
    try {
      generator.generate();
    } catch (final IOException e) {
      throw new RetrascopeException(e);
    }
  }

  @Override
  public void onModuleBegin(final Module module) {

    for (final NodeVariable variable : module.getVariables()) {
      addData("var_names", variable.getName());

      //TODO: use the correct type instead of hard-coded uint64_t
      addData("var_types", "uint64_t");
    }
    addData("num", module.getProcesses().size());
  }
}
