/*
 * Copyright 2019 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package ru.ispras.retrascope.engine.cfg.transformer.gadd.backend;

import ru.ispras.retrascope.model.basis.Statement;
import ru.ispras.retrascope.model.basis.StatementUtils;
import ru.ispras.retrascope.model.cfg.CfgEmptyVisitor;

/**
 * @author <a href="mailto:smolov@ispras.ru">Sergey Smolov</a>
 */
class PathCountVisitor extends CfgEmptyVisitor {

  private int pathNum;

  /**
   * Default constructor.
   */
  public PathCountVisitor() {

    this.pathNum = 0;
  }

  /**
   * Sets the path counter value.
   * @param num The path counter.
   */
  public void setPathNum(final int num) {

    this.pathNum = num;
  }

  /**
   * Returns the calculated path number.
   * @return The calculated path number.
   */
  public int getPathNum() {

    return this.pathNum;
  }

  @Override
  public void onStatementBegin(final Statement statement) {

    if (StatementUtils.isLast(statement)) {
      pathNum++;
    }
  }
}
