/*
 * Copyright 2014-2018 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package ru.ispras.retrascope.engine.efsm.generator.test;

import ru.ispras.fortress.expression.NodeVariable;
import ru.ispras.retrascope.model.efsm.EfsmTransition;

/**
 * The representation of variable dependency between EFSM transitions.
 *
 * @author <a href="mailto:myllyenko@ya.ru">Igor Melnichenko</a>
 */
final class VariableDependency {

  private final EfsmTransition src;
  private final EfsmTransition dst;
  private final NodeVariable var;

  /**
   * Creates a dependency.
   *
   * @param src a transition that affects another one by setting a value of a variable
   * @param dst a transition affected by usage of the set value
   * @param var the variable
   */
  VariableDependency(final EfsmTransition src, final EfsmTransition dst, final NodeVariable var) {
    this.src = src;
    this.dst = dst;
    this.var = var;
  }

  /**
   * Returns a variable which this dependency is related to.
   */
  NodeVariable getVariable() {
    return this.var;
  }

  @Override
  public int hashCode() {
    int result = 1;
    result = 31 * result + ((dst == null) ? 0 : dst.hashCode());
    result = 31 * result + ((src == null) ? 0 : src.hashCode());
    result = 31 * result + ((var == null) ? 0 : var.hashCode());
    return result;
  }

  @Override
  public boolean equals(final Object obj) {
    if (this == obj) {
      return true;
    }
    if (obj == null) {
      return false;
    }
    if (!(obj instanceof VariableDependency)) {
      return false;
    }

    final VariableDependency other = (VariableDependency) obj;
    if (dst == null) {
      if (other.dst != null) {
        return false;
      }
    } else if (!dst.equals(other.dst)) {
      return false;
    }
    if (src == null) {
      if (other.src != null) {
        return false;
      }
    } else if (!src.equals(other.src)) {
      return false;
    }
    if (var == null) {
      return other.var == null;
    } else {
      return var.equals(other.var);
    }
  }
}
