/*
 * Copyright 2018-2019 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package ru.ispras.retrascope.engine.gadd.extractor.property;

import ru.ispras.fortress.expression.NodeVariable;
import ru.ispras.retrascope.basis.Engine;
import ru.ispras.retrascope.basis.Entity;
import ru.ispras.retrascope.basis.EntityType;
import ru.ispras.retrascope.model.basis.walker.WalkerMode;
import ru.ispras.retrascope.model.cfg.CfgDfsWalker;
import ru.ispras.retrascope.model.gadd.GaddModel;
import ru.ispras.retrascope.result.usedef.UseDefContainer;
import ru.ispras.retrascope.result.usedef.UseDefEntity;

import java.util.Map;

/**
 * This engine extracts the use/def properties from the {@link GaddModel} model.
 *
 * @author <a href="mailto:lebedev@ispras.ru">Mikhail Lebedev</a>
 */
public class GaddPropertyExtractor extends Engine {

  public static final String ENGINE_ID = "gadd-property-extractor";

  public static final EntityType OUT_TYPE = UseDefEntity.TYPE;

  /**
   * Constructs a new engine.
   */
  public GaddPropertyExtractor() {
    super(ENGINE_ID);
    addInputType(GaddModel.TYPE);
    setOutputType(OUT_TYPE);
  }

  @Override
  public Entity start(final Map<EntityType, Entity> inputs) {
    final GaddModel gaddModel = (GaddModel) inputs.get(GaddModel.TYPE);
    final GaddUseDefVisitor visitor = new GaddUseDefVisitor();
    final CfgDfsWalker walker = new CfgDfsWalker(visitor, WalkerMode.DFS);
    walker.start(gaddModel.getModel());
    final UseDefEntity result = new UseDefEntity();
    for (Map.Entry<NodeVariable, UseDefContainer> entry : visitor.getUseDefMap().entrySet()) {
      result.addContainer(entry.getKey(), entry.getValue());
    }
    return result;
  }
}
