/*
 * Copyright 2016-2019 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package ru.ispras.retrascope.engine.gadd.transformer.efsm;

/**
 * Auxiliary class for debug information keeping.
 * @author <a href="mailto:smolov@ispras.ru">Sergey Smolov</a>
 */
final class EfsmDebugInfo {
  private int stateNum;
  private int transitionNum;
  private int efsmNum;

  /**
   * Constructs debug info object with the specified parameters.
   * @param stateNum State number.
   * @param transitionNum Transition number.
   * @param efsmNum Model number.
   */
  EfsmDebugInfo(final int stateNum, final int transitionNum, final int efsmNum) {

    this.stateNum = stateNum;
    this.transitionNum = transitionNum;
    this.efsmNum = efsmNum;
  }

  /**
   * Returns state number.
   * @return State number.
   */
  int getStateNum() {
    return this.stateNum;
  }

  /**
   * Sets state number to the specified value.
   * @param stateNum State number.
   */
  void setStateNum(final int stateNum) {
    this.stateNum = stateNum;
  }

  /**
   * Returns model's transition number.
   * @return Transition number.
   */
  int getTransitionNum() {
    return this.transitionNum;
  }

  /**
   * Sets transition number to the specified value.
   * @param transitionNum Transition number.
   */
  void setTransitionNum(final int transitionNum) {
    this.transitionNum = transitionNum;
  }

  /**
   * Returns model number.
   * @return Model number.
   */
  int getModelNum() {
    return this.efsmNum;
  }

  /**
   * Sets model number.
   * @param modelNum Model number.
   */
  void setModelNum(final int modelNum) {
    this.efsmNum = modelNum;
  }
}
