/*
 * Copyright 2019 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package ru.ispras.retrascope.engine.gadd.transformer.hldd.formula;

import ru.ispras.fortress.expression.NodeVariable;

import java.util.Collection;
import java.util.HashSet;
import java.util.LinkedList;
import java.util.List;
import java.util.Set;

/**
 * This class represents a block-level formula builder.
 *
 * @author <a href="mailto:lebedev@ispras.ru">Mikhail Lebedev</a>
 */
final class BlockBuilder {

  // A phase value.
  private int phase;
  private List<Integer> prevPhases;
  private Set<NodeVariable> defines;
  private Set<NodeVariable> uses;

  BlockBuilder(final int phase, final List<Integer> prevPhases) {
    this.phase = phase;
    this.prevPhases = new LinkedList<>(prevPhases);
    defines = new HashSet<>();
    uses = new HashSet<>();
  }

  /**
   * @return The phase variable value of this block.
   */
  int getPhase() {
    return phase;
  }

  List<Integer> getPrevPhases() {
    return prevPhases;
  }

  boolean defines(final NodeVariable variable) {
    return defines.contains(variable);
  }

  boolean uses(final NodeVariable variable) {
    return uses.contains(variable);
  }

  void addDefine(final NodeVariable variable) {
    defines.add(variable);
  }

  void addUse(final NodeVariable variable) {
    uses.add(variable);
  }

  void addUses(final Collection<NodeVariable> variables) {
    for (NodeVariable variable : variables) {
      if (variable.getUserData() instanceof PhasedDescriptor) {
        final PhasedDescriptor descriptor = ((PhasedDescriptor) variable.getUserData());
        if (!descriptor.isGuard()) {
          uses.add(variable);
        }
      } else {
        uses.add(variable);
      }
    }
    //uses.addAll(variables);
  }

  Collection<NodeVariable> getAllUses() {
    return uses;
  }

  Collection<NodeVariable> getAllDefines() {
    return defines;
  }

}
