/*
 * Copyright 2018 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package ru.ispras.retrascope.engine.hldd.printer.smv.basis;

import ru.ispras.fortress.expression.NodeVariable;
import ru.ispras.retrascope.engine.hldd.printer.smv.HlddSmvVisitor;
import ru.ispras.retrascope.engine.hldd.printer.smv.utils.SmvPrinterUtils;
import ru.ispras.retrascope.model.basis.Module;

import java.util.Collection;


/**
 * This is the base class of a visitor for clocked models.
 *
 * @author <a href="mailto:lebedev@ispras.ru">Mikhail Lebedev</a>
 */
public abstract class HlddClockedVisitorBase extends HlddSmvVisitor {

  /**
   * Constructs a new visitor and creates a new *.smv file.
   *
   * @param fileName - the specified *.smv file name.
   */
  public HlddClockedVisitorBase(String fileName) {
    super(fileName);
  }

  @Override
  public void onModuleBegin(final Module hlddModule) {
    super.onModuleBegin(hlddModule);

    // Prints clock and input variables constraints.
    printClock(hlddModule);
  }

  private void printClock(final Module hlddModule) {
    final NodeVariable clock = getClock();
    if (clock != null) {
      append(
          SmvPrinterUtils.getClockAssignment(clock, currentPhaseVars.keySet(), getExprPrinter()));
      appendLine();
      final Collection<NodeVariable> inputs = hlddModule.getInputs();
      inputs.remove(clock);
      append(SmvPrinterUtils.getInputConstraints(inputs, clock));
      appendLine();
    }
  }

  protected abstract NodeVariable getClock();

}
