/*
 * Copyright 2018 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package ru.ispras.retrascope.engine.hldd.printer.smv.basis;

import ru.ispras.retrascope.basis.Arguments;
import ru.ispras.retrascope.basis.Engine;
import ru.ispras.retrascope.basis.EntityType;
import ru.ispras.retrascope.basis.Parameter;

/**
 * Base class for SMV printing engines.
 *
 * @author <a href="mailto:lebedev@ispras.ru">Mikhail Lebedev</a>
 */
public abstract class SmvPrinterEngine extends Engine {

  /**
   * The engine output entity type.
   */
  public static final EntityType OUT_TYPE = EntityType.get("smv");

  /**
   * The command-line parameter.
   */
  public static final Parameter SMV_FILE_NAME =
      new Parameter("smv-file", true, "Output SMV file name");

  private static final String DEFAULT_FILE_NAME = "model.smv";

  protected SmvPrinterEngine(final String engineId) {
    super(engineId);
    setOutputType(OUT_TYPE);
    addParameter(SMV_FILE_NAME);
  }

  protected String getSmvFileName(final Arguments arguments) {
    return arguments.getValue(SMV_FILE_NAME, DEFAULT_FILE_NAME);
  }

}
