/*
 * Copyright 2016 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package ru.ispras.retrascope.engine.hldd.printer.smv.property;

import ru.ispras.retrascope.basis.Arguments;
import ru.ispras.retrascope.basis.Entity;
import ru.ispras.retrascope.basis.EntityType;
import ru.ispras.retrascope.basis.Parameter;
import ru.ispras.retrascope.engine.hldd.printer.smv.basis.HlddSmvPrinterBase;
import ru.ispras.retrascope.engine.hldd.printer.smv.basis.ModelCheckingMethod;
import ru.ispras.retrascope.model.hldd.HlddVisitor;
import ru.ispras.retrascope.result.property.PropertyEntity;

import java.util.Map;

/**
 * This engine prints the HLDD model and related properties into an SMV file.
 *
 * @author <a href="mailto:lebedev@ispras.ru">Mikhail Lebedev</a>
 */
public final class HlddPropertySmvPrinter extends HlddSmvPrinterBase {

  /**
   * The engine ID.
   */
  public static final String ENGINE_ID = "hldd-property-smv-printer";

  /**
   * The model checking method command line parameter.
   */
  public static final Parameter CHECK_METHOD =
      new Parameter("check-method", ModelCheckingMethod.getStringValues(), "Model checking method");

  /**
   * Default constructor.
   */
  public HlddPropertySmvPrinter() {
    super(ENGINE_ID);
    addInputType(PropertyEntity.TYPE);
    addParameter(CHECK_METHOD);
  }

  @Override
  protected HlddVisitor getVisitor(String fileName) {
    return new HlddPropertySmvVisitor(fileName);
  }

  @Override
  protected void configureVisitor(Map<EntityType, Entity> inputs, Arguments args,
      HlddVisitor visitor) {
    final String checkType =
        args.getValue(CHECK_METHOD, ModelCheckingMethod.LTL_BMC.getStringValue())
            .toLowerCase();
    ((HlddPropertySmvVisitor) visitor).setSpecType(checkType);
    if (inputs.containsKey(PropertyEntity.TYPE)) {
      PropertyEntity propertyEntity = (PropertyEntity) inputs.get(PropertyEntity.TYPE);
      ((HlddPropertySmvVisitor) visitor).addAssertions(propertyEntity.getContainerMap());
    }
  }
}
