/*
 * Copyright 2018-2019 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package ru.ispras.retrascope.engine.hldd.printer.smv.usedef;

import ru.ispras.retrascope.basis.Arguments;
import ru.ispras.retrascope.basis.Entity;
import ru.ispras.retrascope.basis.EntityType;
import ru.ispras.retrascope.engine.hldd.printer.smv.basis.HlddSmvPrinterBase;
import ru.ispras.retrascope.model.gadd.GaddModel;
import ru.ispras.retrascope.model.hldd.HlddModel;
import ru.ispras.retrascope.model.hldd.HlddVisitor;

import java.util.Map;

/**
 * This class prints the HLDD model with the use-def checkers and properties.
 *
 * @author <a href="mailto:lebedev@ispras.ru">Mikhail Lebedev</a>
 */
public class HlddUseDefSmvPrinter extends HlddSmvPrinterBase {

  public static final String ENGINE_ID = "hldd-usedef-smv-printer";

  /**
   * Constructs a new printer engine.
   */
  public HlddUseDefSmvPrinter() {
    super(ENGINE_ID);
    addInputType(GaddModel.TYPE);
  }

  @Override
  protected HlddVisitor getVisitor(final String fileName) {
    return new HlddUseDefSmvVisitor(fileName);
  }

  @Override
  protected void configureVisitor(final Map<EntityType, Entity> inputs, final Arguments args,
      final HlddVisitor visitor) {

    if (inputs.containsKey(GaddModel.TYPE) && inputs.containsKey(HlddModel.TYPE)) {

      final GaddModel model = (GaddModel) inputs.get(GaddModel.TYPE);
      final HlddModel hlddModel = (HlddModel) inputs.get(HlddModel.TYPE);

      ((HlddUseDefSmvVisitor) visitor).setGaddModel(model);
      ((HlddUseDefSmvVisitor) visitor).setUseDefMap(hlddModel.getUseDefMap());
    }
  }

}
