/*
 * Copyright 2015-2019 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package ru.ispras.retrascope.engine.smv.test.parser;

import ru.ispras.retrascope.basis.Engine;
import ru.ispras.retrascope.basis.Entity;
import ru.ispras.retrascope.basis.EntityType;
import ru.ispras.retrascope.basis.FileNames;
import ru.ispras.retrascope.basis.exception.RetrascopeException;
import ru.ispras.retrascope.engine.efsm.generator.test.TestMinimiser;
import ru.ispras.retrascope.engine.smv.launcher.SmvModelCheckerLauncher;
import ru.ispras.retrascope.engine.smv.test.parser.parsers.SmvCounterexampleParser;
import ru.ispras.retrascope.model.basis.EventList;
import ru.ispras.retrascope.model.basis.Module;
import ru.ispras.retrascope.model.basis.ModuleProcess;
import ru.ispras.retrascope.model.gadd.GaddModel;
//import ru.ispras.retrascope.model.efsm.EfsmModel;
import ru.ispras.retrascope.result.cfginterface.CfgInterface;
//import ru.ispras.retrascope.result.test.Sequence;
import ru.ispras.retrascope.result.test.Test;
import ru.ispras.retrascope.result.test.TestModel;
import ru.ispras.retrascope.result.test.TestVisitor;
import ru.ispras.retrascope.result.test.TestWalker;

import java.util.Map;

/**
 * This engine parses the *.smvlog files generated by the model checker and constructs a new
 * {@link TestModel} entity.
 *
 * @author <a href="mailto:lebedev@ispras.ru">Mikhail Lebedev</a>
 */
public final class SmvTestParser extends Engine {

  /** The engine ID. */
  public static final String ENGINE_ID = "smv-test-parser";

  /**
   * Constructs a new parser object.
   */
  public SmvTestParser() {
    super(ENGINE_ID);

    addInputType(SmvModelCheckerLauncher.OUT_TYPE);
    addInputType(CfgInterface.TYPE);
    addInputType(GaddModel.TYPE);
    //addInputType(EfsmModel.TYPE);
    setOutputType(TestModel.TYPE);
  }

  @Override
  public Entity start(Map<EntityType, Entity> inputs) {

    final TestModel result = new TestModel();

    //final EfsmModel efsmModel = Engine.getInputEntity(inputs, EfsmModel.class, EfsmModel.TYPE);

    try {

      final FileNames files = (FileNames) inputs.get(SmvModelCheckerLauncher.OUT_TYPE);
      final CfgInterface iface = (CfgInterface) inputs.get(CfgInterface.TYPE);
      final GaddModel gadd = (GaddModel) inputs.get(GaddModel.TYPE);
      final EventList events = new EventList();

      for (final Module module : gadd.getModel().getModules()) {
        for (final ModuleProcess process : module.getProcesses()) {
          final EventList list = process.getEventList();
          events.addEvents(list);
        }
      }

      final SmvCounterexampleParser parser = new SmvCounterexampleParser();
      parser.setSignalInterface(iface);
      parser.setEventList(events);

      if (files != null) {
        for (String fileName : files.getList()) {
          final Test test = parser.parseCounterexamples(fileName);
          result.addTest(test);
        }
      }

    } catch (final Exception e) {
      throw new RetrascopeException(e);
    }

    final TestModel minTest = TestMinimiser.minimise(result/*, efsmModel*/);

    return removeExhaustiveEvents(minTest.isEmpty() ? result : minTest);
  }

  /*private TestModel merge(final TestModel model) {

    *//* Remove events from test model since they're already stored in test data. *//*
    removeExhaustiveEvents(model);
    return mergeSequences(model);
  }

  private TestModel mergeSequences(final TestModel testModel) {
    final Sequence mergedSequence = new Sequence();
    for (final Test test : testModel.getTests()) {
      for (final Sequence sequence : test.getSequences()) {
        sequence.initIterator();
        while (sequence.hasNext()) {
          mergedSequence.addVector(sequence.next());
        }
      }
    }
    final Test mergedTest = new Test();
    mergedTest.addSequence(mergedSequence);
    return new TestModel(mergedTest);
  }*/

  private TestModel removeExhaustiveEvents(final TestModel testModel) {

    final TestVisitor removeEventVisitor = new RmEventVisitor();

    for (final Test test : testModel.getTests()) {
      TestWalker walker = new TestWalker(removeEventVisitor);
      walker.start(test);
    }
    return testModel;
  }
}
