/*
 * Copyright 2016 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package ru.ispras.retrascope.engine.smv.test.parser.parsers;

import java.util.regex.Matcher;

/**
 * This class is used to parse an SMV log file and extract counterexample statistics.
 *
 * @author <a href="mailto:lebedev@ispras.ru">Mikhail Lebedev</a>
 */
public class SmvStatisticsParser extends LogParser {

  /* The number of "true" specifications. */
  private int trueSpecsCount;
  /* The number of counterexamples. */
  private int counterexamplesCount;

  /**
   * Constructs a new parser.
   */
  public SmvStatisticsParser() {
    super();
    trueSpecsCount = 0;
  }

  /**
   * Parses checker output for run statistics.
   *
   * @param fileName - the specified *.smvlog file name.
   */
  public void parseStats(final String fileName) {
    init();
    parse(fileName);
  }

  @Override
  public void parseLine(String line) {
    final Matcher trueMatcher = trueSpec.matcher(line);
    final Matcher specMatcher = counterexample.matcher(line);

    if (trueMatcher.find()) {
      trueSpecsCount++;
    }
    if (specMatcher.find()) {
      counterexamplesCount++;
    }
  }

  /**
   * Returns the number of true specifications after model checking.
   *
   * @return The number of true specifications.
   */
  public int getTrueSpecs() {
    return trueSpecsCount;
  }

  /**
   * Returns the number of counterexamples generated during model checking.
   *
   * @return The number of counterexamples.
   */
  public int getCounterexamples() {
    return counterexamplesCount;
  }

  private void init() {
    trueSpecsCount = 0;
    counterexamplesCount = 0;
  }
}
