/*
 * Copyright 2016-2018 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package ru.ispras.retrascope.engine.test.printer.testbench.verilog;

import ru.ispras.fortress.data.DataType;
import ru.ispras.retrascope.basis.Entity;
import ru.ispras.retrascope.basis.EntityType;
import ru.ispras.retrascope.engine.test.printer.testbench.TestbenchPrinterEngine;
import ru.ispras.retrascope.result.testbench.VerilogTestbenchModel;

import java.util.LinkedHashMap;
import java.util.Map;

/**
 * Test printer to Verilog testbench format.
 * @author <a href="mailto:smolov@ispras.ru">Sergey Smolov</a>
 */
public class TestVerilogTestbenchPrinter extends TestbenchPrinterEngine {

  /**
   * The engine identifier.
   */
  public static final String ENGINE_ID = "test-verilog-testbench-printer";

  private static final Map<DataType, String> DATA_TYPES_MAP = new LinkedHashMap<>();

  /**
   * Creates a new {@code TestVhdlTestbenchPrinter} instance.
   *
   */
  public TestVerilogTestbenchPrinter() {
    super(ENGINE_ID, VerilogTestbenchModel.TYPE);
  }

  static {
    DATA_TYPES_MAP.put(DataType.bitVector(1), "");
    DATA_TYPES_MAP.put(DataType.INTEGER, "");
    DATA_TYPES_MAP.put(DataType.BOOLEAN, "");
    DATA_TYPES_MAP.put(DataType.REAL, "");
  }

  @Override
  public String getTestbenchExt() {
    return "v";
  }

  @Override
  protected String getTemplateDirName() {
    return "stg/verilog/";
  }

  @Override
  protected String getTemplateName() {
    return "VerilogTestbench.stg";
  }

  @Override
  protected String getTstParserName() {
    return null;
  }

  @Override
  protected String getStEntityName() {
    return "verilog_testbench";
  }

  @Override
  public String getInputPortKind() {
    return null;
  }

  @Override
  public String getOutputPortKind() {
    return null;
  }

  @Override
  public String getBitVectorPrefix() {
    return "[";
  }

  @Override
  public String getBitVectorDelim() {
    return ":";
  }

  @Override
  public String getBitVectorSuffix() {
    return "]";
  }

  @Override
  public String getInputDeclarationFmt() {
    return "reg";
  }

  @Override
  public String getOutputDeclarationFmt() {
    return "wire";
  }

  @Override
  public Map<DataType, String> getDataTypeMap() {
    return DATA_TYPES_MAP;
  }

  @Override
  public Map<String, String> getConvertFunctionMap() {
    return new LinkedHashMap<>();
  }

  @Override
  public Entity start(Map<EntityType, Entity> inputs) {

    return new VerilogTestbenchModel(makeTestbenches(inputs));
  }
}
