/*
 * Copyright 2014-2016 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package ru.ispras.retrascope.model.cfg;

import ru.ispras.retrascope.model.basis.walker.ModelVisitor;

/**
 * Control flow graph model visitor.
 *
 * @author <a href="mailto:smolov@ispras.ru">Sergey Smolov</a>
 */
public interface CfgVisitor extends ModelVisitor {

  /**
   * Starts visiting {@link CfgAssertStatement} node.
   *
   * @param assertStatement Assert statement to be visited.
   */
  void onAssertBegin(final CfgAssertStatement assertStatement);

  /**
   * Finishes visiting {@link CfgAssertStatement} node.
   *
   * @param assertStatement Assert statement to be visited.
   */
  void onAssertEnd(final CfgAssertStatement assertStatement);

  /**
   * Starts visiting {@link CfgBlockStatement} node.
   *
   * @param block Block statement to be visited.
   */
  void onBlockBegin(final CfgBlockStatement block);

  /**
   * Finishes visiting {@link CfgBlockStatement} node.
   *
   * @param block Block statement to be visited.
   */
  void onBlockEnd(final CfgBlockStatement block);

  /**
   * Starts visiting {@link CfgCaseStatement} node.
   *
   * @param caseNode Case statement to be visited.
   */
  void onCaseBegin(final CfgCaseStatement caseNode);

  /**
   * Finishes visiting {@link CfgCaseStatement} node.
   *
   * @param caseNode Case statement to be visited.
   */
  void onCaseEnd(final CfgCaseStatement caseNode);

  /**
   * Starts visiting {@link CfgSwitchStatement} node.
   *
   * @param switchNode Switch statement to be visited.
   */
  void onSwitchBegin(final CfgSwitchStatement switchNode);

  /**
   * Finishes visiting {@link CfgSwitchStatement} node.
   *
   * @param switchNode Switch statement to be visited.
   */
  void onSwitchEnd(final CfgSwitchStatement switchNode);

  /**
   * Starts visiting {@link CfgLoopStatement} node.
   *
   * @param loopStatement Loop statement to be visited.
   */
  void onLoopBegin(final CfgLoopStatement loopStatement);

  /**
   * Finishes visiting {@link CfgLoopStatement} node.
   *
   * @param loopStatement Loop statement to be visited.
   */
  void onLoopEnd(final CfgLoopStatement loopStatement);

  /**
   * Starts visiting {@link CfgWaitStatement} node.
   *
   * @param wait Wait statement to be visited.
   */
  void onWaitBegin(final CfgWaitStatement wait);

  /**
   * Finishes visiting {@link CfgWaitStatement} node.
   *
   * @param wait Wait statement to be visited.
   */
  void onWaitEnd(final CfgWaitStatement wait);
}
