/*
 * Copyright 2015-2017 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package ru.ispras.retrascope.model.efsm;

import ru.ispras.retrascope.model.basis.State;

import java.util.ArrayList;
import java.util.Collection;

/**
 * A state of an extended finite state machine module.
 *
 * @author <a href="mailto:i.melnichenko@deltasolutions.ru">Igor Melnichenko</a>
 */
public class EfsmModuleState extends State {
  private final Collection<State> substates;

  /**
   * Constructs an EFSM module state on the basis of states of the module's components.
   *
   * @param substates the substates
   */
  public EfsmModuleState(final Collection<State> substates) {
    super();
    this.substates = new ArrayList<>(substates);
  }

  @Override
  public String getId() {
    return this.toString();
  }

  @Override
  public int hashCode() {
    return 31 * super.hashCode() + this.substates.hashCode();
  }

  @Override
  public boolean equals(final Object object) {

    if (this == object) {
      return true;
    }

    if (!super.equals(object)) {
      return false;
    }

    if (!(object instanceof EfsmModuleState)) {
      return false;
    }

    final EfsmModuleState other = (EfsmModuleState) object;

    return this.substates.equals(other.substates);
  }

  @Override
  public String toString() {
    return this.substates.toString();
  }
}
