/*
 * Copyright 2014-2016 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package ru.ispras.retrascope.model.efsm;

import ru.ispras.retrascope.model.basis.walker.ModelVisitor;

/**
 * The interface for EFSM visitors. Typically, EFSM visitors are used along with EFSM walkers.
 *
 * @author <a href="mailto:i.melnichenko@deltasolutions.ru">Igor Melnichenko</a>
 *
 * @see Efsm
 * @see EfsmDfsWalker
 */
public interface EfsmVisitor extends ModelVisitor {

  /**
   * Indicates that processing of the specified EFSM state has been started.
   *
   * @param state the processed EFSM state
   */
  void onStateBegin(final EfsmState state);

  /**
   * Indicates that processing of the specified EFSM state has been finished.
   *
   * @param state the processed EFSM state
   */
  void onStateEnd(final EfsmState state);

  /**
   * Indicates that processing of the specified EFSM transition has been started.
   *
   * @param transition the processed EFSM transition
   */
  void onTransitionBegin(final EfsmTransition transition);

  /**
   * Indicates that processing of the specified EFSM transition has been finished.
   *
   * @param transition the processed EFSM transition
   */
  void onTransitionEnd(final EfsmTransition transition);

  /**
   * Sets internal status field to the specified value.
   *
   * @param status The status value to be set.
   */
  void setStatus(final Status status);
}
