/*
 * Copyright 2017-2019 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package ru.ispras.retrascope.model.gadd;

import java.util.Objects;

/**
 * Phase of {@link GaddModel} model path.
 * @author <a href="mailto:smolov@ispras.ru">Sergey Smolov</a>
 */
public class Phase {

  private final Integer intPhase;

  /**
   * Creates phase object with the specified phase value.
   * @param phase The phase value
   */
  public Phase(final Integer phase) {
    this.intPhase = phase;
  }

  /**
   * Returns the phase value.
   * @return The phase value.
   */
  public Integer getValue() {
    return intPhase;
  }

  @Override
  public String toString() {
    return getValue().toString();
  }

  @Override
  public boolean equals(final Object obj) {

    if (this == obj) {
      return true;
    }
    if (obj == null || getClass() != obj.getClass()) {
      return false;
    }

    final Phase thatPhase = (Phase) obj;

    return Objects.equals(intPhase, thatPhase.intPhase);
  }

  @Override
  public int hashCode() {
    return intPhase == null ? 0 : intPhase.hashCode();
  }
}
