/*
 * Copyright 2015-2016 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package ru.ispras.retrascope.model.hldd;

import ru.ispras.fortress.expression.NodeVariable;
import ru.ispras.retrascope.basis.Entity;
import ru.ispras.retrascope.basis.EntityType;
import ru.ispras.retrascope.model.basis.Model;
import ru.ispras.retrascope.model.basis.Module;
import ru.ispras.retrascope.result.usedef.UseDefContainer;

import java.util.Collection;
import java.util.Map;

/**
 * This class represents a HlddModel which consists of HlddModules, HlddProcesses and Hldds.
 *
 * @author <a href="mailto:lebedev@ispras.ru">Mikhail Lebedev</a>
 */
public final class HlddModel extends Model {

  // Entity type.
  public static final EntityType TYPE = EntityType.get("hldd");

  private Map<NodeVariable, UseDefContainer> useDefMap;

  /**
   * Constructs a new model entity.
   */
  public HlddModel() {
    super(TYPE);
  }

  /**
   * Constructs a new model entity using the specified collection of modules.
   *
   * @param modules - the specified collection of {@link HlddModule}.
   */
  public HlddModel(final Collection<Module> modules) {
    super(TYPE);
    addModules(modules);
  }

  @Override
  public void add(final Entity entity) {
    if (entity instanceof HlddModel) {
      addModules(((HlddModel) entity).getModules());
      setUseDefMap(((HlddModel) entity).getUseDefMap());
    } else {
      throw new IllegalArgumentException();
    }
  }

  public Map<NodeVariable, UseDefContainer> getUseDefMap() {
    return useDefMap;
  }

  public void setUseDefMap(final Map<NodeVariable, UseDefContainer> useDefMap) {
    this.useDefMap = useDefMap;
  }
}
