/*
 * Copyright 2016 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package ru.ispras.retrascope.model.hldd;

import ru.ispras.retrascope.model.basis.Module;
import ru.ispras.retrascope.util.Log;

/**
 * Utility methods for high level decision diagram (HLDD) model and it's components.
 *
 * @author <a href="mailto:smolov@ispras.ru">Sergey Smolov</a>
 */
public final class HlddModelUtils {

  private HlddModelUtils() {
    /* Empty. */
  }

  /**
   * Checks whether the model contains empty modules.
   *
   * @param model The model to be checked.
   * @return {@code true} if at least one module of the specified model is empty, {@code false}
   *         otherwise.
   */
  public static boolean containsEmptyModules(final HlddModel model) {
    if (model == null) {
      return true;
    }

    for (Module module : model.getModules()) {
      if (module.getName().isEmpty()) {
        Log.warn("Empty module found!");
        return true;
      }
    }
    return false;
  }
}
