/*
 * Copyright 2017 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package ru.ispras.retrascope.parser.basis.backend;

import ru.ispras.fortress.expression.NodeValue;
import ru.ispras.retrascope.model.cfg.CfgCaseStatement;
import ru.ispras.retrascope.model.cfg.CfgEmptyVisitor;
import ru.ispras.retrascope.model.cfg.CfgSwitchStatement;

import java.util.Collection;
import java.util.LinkedList;

/**
 * Parser backend that merges empty case statements of one switch statement.
 *
 * @author <a href="mailto:smolov@ispras.ru">Sergey Smolov</a>
 */
public class CfgEmptyCaseBackend extends CfgEmptyVisitor {

  /**
   * The backend name.
   */
  public static final String NAME = "empty_case";

  @Override
  public void onSwitchBegin(final CfgSwitchStatement switchNode) {

    final Collection<CfgCaseStatement> emptyCases = new LinkedList<>();
    final int switchStmtNum = switchNode.getStatementNum();

    /* Collapse empty cases that are free of events. */
    for (int i = 0; i < switchStmtNum; i++) {

      final CfgCaseStatement caseStatement = (CfgCaseStatement) switchNode.getStatement(i);

      if (!caseStatement.containStatements() && !caseStatement.hasEvents()) {
        emptyCases.add(caseStatement);
      }
    }

    if (emptyCases.size() > 1) {

      final Collection<NodeValue> values = new LinkedList<>();
      boolean isDefault = false;

      for (final CfgCaseStatement emptyCase : emptyCases) {

        switchNode.remove(emptyCase);
        values.addAll(emptyCase.getValues());

        if (emptyCase.isDefault()) {
          isDefault = true;
        }
      }
      final CfgCaseStatement onlyCase =
          (isDefault || values.isEmpty())
              ? new CfgCaseStatement(true)
              : new CfgCaseStatement(values);

      switchNode.add(onlyCase);
    }
  }
}
