/*
 * Copyright 2019 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package ru.ispras.retrascope.parser.basis.backend;

import ru.ispras.retrascope.basis.Entity;
import ru.ispras.retrascope.engine.cfg.CfgVisitorBackend;
import ru.ispras.retrascope.model.basis.walker.WalkerMode;
import ru.ispras.retrascope.util.Log;

/**
 * Backend for the HDL parser that gathers statistic information
 * about the module under consideration.
 * @author <a href="mailto:smolov@ispras.ru">Sergey Smolov</a>
 */
public class CfgStatisticBackend extends CfgVisitorBackend {

  /**
   * The backend name.
   */
  public static final String NAME = "statistic";

  /* Debug printing output format. */
  private static final String DEBUG_FMT = "engine=hdl-parser modules=%d variables=%d processes=%d"
      + " switches=%d blocks=%d\n";

  public CfgStatisticBackend() {
    super(NAME, new CfgStatisticVisitor(), WalkerMode.DFS_NO_RPT);
  }

  @Override
  public void start(final Entity entity) {

    super.start(entity);

    final int moduleNum = ((CfgStatisticVisitor) getVisitor()).getModuleNum();
    final int varNum = ((CfgStatisticVisitor) getVisitor()).getVarNum();
    final int processNum = ((CfgStatisticVisitor) getVisitor()).getProcessNum();
    final int switchNum = ((CfgStatisticVisitor) getVisitor()).getSwitchNum();
    final int blockNum = ((CfgStatisticVisitor) getVisitor()).getBlockNum();

    final String statistics =
        String.format(DEBUG_FMT, moduleNum, varNum, processNum, switchNum, blockNum);

    Log.debug(statistics);
    debugWrite(statistics);
  }
}
