/*
 * Copyright 2019 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package ru.ispras.retrascope.parser.basis.backend;

import ru.ispras.retrascope.model.basis.Model;
import ru.ispras.retrascope.model.basis.Module;
import ru.ispras.retrascope.model.basis.ModuleProcess;
import ru.ispras.retrascope.model.basis.Statement;
import ru.ispras.retrascope.model.cfg.CfgBlockStatement;
import ru.ispras.retrascope.model.cfg.CfgCaseStatement;
import ru.ispras.retrascope.model.cfg.CfgEmptyVisitor;
import ru.ispras.retrascope.model.cfg.CfgSwitchStatement;

/**
 * Visitor for CFG model that gathers some statistical information.
 * @author <a href="mailto:smolov@ispras.ru">Sergey Smolov</a>
 */
final class CfgStatisticVisitor extends CfgEmptyVisitor {

  private int moduleNum;

  private int processNum;

  private int switchNum;

  private int blockNum;

  private int varNum;

  private boolean insideProcess;

  int getModuleNum() {
    return this.moduleNum;
  }

  int getProcessNum() {
    return this.processNum;
  }

  int getSwitchNum() {
    return this.switchNum;
  }

  int getBlockNum() {
    return this.blockNum;
  }

  int getVarNum() {
    return this.varNum;
  }

  @Override
  public void onModelBegin(final Model model) {

    this.moduleNum = 0;
    this.processNum = 0;
    this.switchNum = 0;
    this.blockNum = 0;
    this.varNum = 0;
  }

  @Override
  public void onStatementBegin(final Statement statement) {

    final boolean isBlock = statement instanceof CfgBlockStatement;
    final boolean isSwitch = statement instanceof CfgSwitchStatement;

    if (this.insideProcess && (isBlock || isSwitch)) {

      if (isBlock) {
        this.blockNum++;
      } else if (isSwitch) {
        this.switchNum++;
      }
    }
  }

  @Override
  public void onModuleBegin(final Module module) {
    this.moduleNum++;
    this.varNum += module.getVariables().size();
  }

  @Override
  public void onProcessBegin(final ModuleProcess process) {

    this.processNum++;
    this.insideProcess = true;
  }

  @Override
  public void onProcessEnd(final ModuleProcess process) {

    this.insideProcess = false;
  }
}
