/*
 * Copyright 2015-2016 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package ru.ispras.retrascope.result.property;

import ru.ispras.fortress.util.InvariantChecks;
import ru.ispras.retrascope.basis.Entity;
import ru.ispras.retrascope.basis.EntityType;
import ru.ispras.retrascope.basis.exception.RetrascopeException;
import ru.ispras.retrascope.util.LogMessage;

import java.util.Collections;
import java.util.LinkedHashMap;
import java.util.Map;

/**
 * Entity that contains properties that are extracted from models.
 *
 * @author <a href="mailto:smolov@ispras.ru">Sergey Smolov</a>
 */
public class PropertyEntity extends Entity {

  public static final EntityType TYPE = EntityType.get("property");

  private final Map<String, ModuleProperty> containerMap;

  /**
   * Default constructor.
   */
  public PropertyEntity() {
    super(TYPE);

    this.containerMap = new LinkedHashMap<>();
  }

  /**
   * Returns module-level property containers mapping.
   * <p>Container names are stored in mapping keys.</p>
   * @return Mapping of names to module-level property containers.
   */
  public Map<String, ModuleProperty> getContainerMap() {

    return Collections.unmodifiableMap(this.containerMap);
  }

  /**
   * Adds non-null module-level property container.
   *
   * @param property Module level property container to be added.
   *
   * @throws IllegalArgumentException when argument is {@code null}.
   */
  public void addContainer(final ModuleProperty property) {
    InvariantChecks.checkNotNull(property);

    this.containerMap.put(property.getName(), property);
  }

  @Override
  public void add(final Entity entity) {

    if (entity instanceof PropertyEntity) {

      this.containerMap.putAll(((PropertyEntity) entity).getContainerMap());
    } else {
      throw RetrascopeException.exception(LogMessage.WRONG_TYPE, entity.getEntityType());
    }
  }
}
