/*
 * Copyright 2015-2017 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package ru.ispras.retrascope.result.property;

/**
 * Property entity visitor.
 * @author <a href="mailto:smolov@ispras.ru">Sergey Smolov</a>
 */
interface PropertyVisitor {

  /**
   * Notifies that visiting process has been started.
   */
  void onRootBegin();

  /**
   * Notifies that visiting process has been finished.
   */
  void onRootEnd();

  /**
   * Starts visiting of the specified entity object.
   *
   * @param entity Entity to be visited.
   */
  void onEntityBegin(final PropertyEntity entity);

  /**
   * Finishes visiting of the specified entity object.
   *
   * @param entity Entity that has been visited.
   */
  void onEntityEnd(final PropertyEntity entity);

  /**
   * Starts visiting of the specified module-level property container object.
   *
   * @param moduleProperty Module-level property container to be visited.
   */
  void onContainerBegin(final ModuleProperty moduleProperty);

  /**
   * Finishes visiting of the specified module-level property container object.
   *
   * @param moduleProperty Module-level property container that has been visited.
   */
  void onContainerEnd(final ModuleProperty moduleProperty);

  /**
   * Notifies that process-level property group has been visited.
   *
   * @param processProperty Process-level property group that has been visited.
   */
  void onProcessProperty(final ProcessProperty processProperty);
}
