/*
 * Copyright 2018 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package ru.ispras.retrascope.result.test;

import java.util.Collection;
import java.util.LinkedHashSet;

/**
 * Utility methods for {@link TestModel} class and it's components.
 * @author <a href="mailto:smolov@ispras.ru">Sergey Smolov</a>
 */
public class TestModelUtils {

  private TestModelUtils() {
    /* Empty. */
  }

  /**
   * Returns the collection of objects from the specified collection that have common sub-list
   * at the beginning of the specified object.
   * <p>The common sub-list for two sequences means that one sequence is completely included
   * into another one, moreover, it coincides with the beginning of the another sequence.</p>
   * @param sequences The collection to be checked.
   * @param sequence The object with which other are compared.
   * @return The collection of objects from the specified collection that have common sub-list
   *         at the beginning of the specified object.
   */
  public static Collection<Sequence> getBeginSubListSequences(
      final Collection<Sequence> sequences,
      final Sequence sequence) {

    final Collection<Sequence> commonStarted = new LinkedHashSet<>();

    for (final Sequence another : sequences) {

      if (another.includes(sequence) || sequence.includes(another)) {
        commonStarted.add(another);
      }
    }

    return commonStarted;
  }
}
