/*
 * Copyright 2018 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package ru.ispras.retrascope.result.usedef;

import ru.ispras.fortress.expression.Node;
import ru.ispras.retrascope.model.basis.ModuleProcess;

import java.util.HashMap;
import java.util.Map;
import java.util.Set;

/**
 * @author <a href="mailto:lebedev@ispras.ru">Mikhail Lebedev</a>
 */
public class UseDefContainer {

  private Map<String, Node> defines;
  private Map<String, Node> uses;

  public UseDefContainer() {
    defines = new HashMap<>();
    uses = new HashMap<>();
  }

  public boolean isDefined(final ModuleProcess process) {
    return compareKeys(defines.keySet(), process.getId());
  }

  public boolean isUsed(final ModuleProcess process) {
    return compareKeys(uses.keySet(), process.getId());
  }

  private boolean compareKeys(final Set<String> keys, final String name) {
    for (String key : keys) {
      if (key.compareTo(name) == 0) {
        return true;
      }
    }
    return false;
  }

  public Map<String, Node> getDefines() {
    return defines;
  }

  public Map<String, Node> getUses() {
    return uses;
  }

  public Set<String> getDefProcesses() {
    return defines.keySet();
  }

  public Set<String> getUseProcesses() {
    return uses.keySet();
  }

  public void putDefineCondition(final ModuleProcess process, final Node node) {
    defines.put(process.getId(), node);
  }

  public void putUseCondition(final ModuleProcess process, final Node node) {
    uses.put(process.getId(), node);
  }

  public Node getDefineCondition(final String processId) {
    return defines.get(processId);
  }

  public Node getDefineCondition(final ModuleProcess process) {
    return getDefineCondition(process.getId());
  }

  public Node getUseCondition(final String processId) {
    return uses.get(processId);
  }

  public Node getUseCondition(final ModuleProcess process) {
    return getUseCondition(process.getId());
  }

}
