/*
 * Copyright 2018 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package ru.ispras.retrascope.result.usedef;

import ru.ispras.fortress.expression.NodeVariable;
import ru.ispras.fortress.util.InvariantChecks;
import ru.ispras.retrascope.basis.Entity;
import ru.ispras.retrascope.basis.EntityType;
import ru.ispras.retrascope.basis.exception.RetrascopeException;
import ru.ispras.retrascope.util.LogMessage;

import java.util.HashMap;
import java.util.Map;

/**
 * This class represents the entity that represents the use/def conditions of the variables.
 *
 * @author <a href="mailto:lebedev@ispras.ru">Mikhail Lebedev</a>
 */
public class UseDefEntity extends Entity {

  public static final EntityType TYPE = EntityType.get("use-def");

  private Map<NodeVariable, UseDefContainer> useDefMap;

  /**
   * Constructs an entity with a given type.
   */
  public UseDefEntity() {
    super(TYPE);
    useDefMap = new HashMap<>();
  }

  /**
   * Returs the variables and their corresponding use/def conditions contained in the entity.
   *
   * @return The variable-use/def container map.
   */
  public Map<NodeVariable, UseDefContainer> getUseDefMap() {
    return useDefMap;
  }

  /**
   * Adds the use/def conditions for the specified variable.
   *
   * @param variable - the specified variable.
   * @param container - the specified use/def conditions container.
   */
  public void addContainer(final NodeVariable variable, UseDefContainer container) {
    InvariantChecks.checkNotNull(variable);
    InvariantChecks.checkNotNull(container);

    useDefMap.put(variable, container);
  }

  @Override
  public void add(final Entity entity) {

    if (entity instanceof UseDefEntity) {
      useDefMap.putAll(((UseDefEntity) entity).getUseDefMap());
    } else {
      throw RetrascopeException.exception(LogMessage.WRONG_TYPE, entity.getEntityType());
    }
  }
}
