/*
 * Copyright 2014-2017 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package ru.ispras.retrascope;

import org.junit.Before;
import ru.ispras.fortress.solver.Solver;
import ru.ispras.fortress.solver.SolverId;

import java.util.Properties;

/**
 * Basic class for tests that use SMT solver without Retrascope running.
 *
 * @author <a href="mailto:smolov@ispras.ru">Sergey Smolov</a>
 */
public class SolverUtilTest {

  /**
   * Default constructor.
   */
  protected SolverUtilTest() {
    /* Empty. */
  }

  /**
   * Configures SMT solver.
   */
  @Before
  public void setEnvironment() {

    final Properties properties = System.getProperties();
    String solverPath = properties.getProperty(Retrascope.SOLVER_PROPERTY);

    if (solverPath == null) {
      solverPath = System.getenv(Retrascope.SOLVER_VARIABLE);

      if (solverPath == null) {
        throw new IllegalStateException(
            String.format(
                "Neither %s nor %s is set.",
                Retrascope.SOLVER_PROPERTY,
                Retrascope.SOLVER_VARIABLE));
      }
    }
    final Solver solver = SolverId.Z3_TEXT.getSolver();
    solver.setSolverPath(solverPath);
  }
}
