/*
 * Copyright 2018 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package ru.ispras.retrascope;

import ru.ispras.retrascope.util.ToolTest;

import java.nio.file.Path;
import java.nio.file.Paths;

/**
 * Basic class for jUnit tests that use VHDL modules from project test suite.
 *
 * @author <a href="mailto:smolov@ispras.ru">Sergey Smolov</a>
 */
public abstract class VhdlTest extends ToolTest {

  private static String getVhdlFileDir() {

    return Paths.get(System.getenv("TOOL_DIR_PATH"), "src", "test", "vhdl").toString();
  }

  /**
   * Returns the module-specific sub-path.
   * @return The module-specific sub-path.
   */
  protected abstract Path[] getModulePaths();

  /**
   * Returns the full path to target module.
   * @return The full path to target module.
   */
  protected String[] getPaths() {

    final Path[] modulePaths = getModulePaths();
    final String[] paths = new String[modulePaths.length];

    for (int i = 0; i < modulePaths.length; i++) {
      paths[i] = Paths.get(getVhdlFileDir(), modulePaths[i].toString()).toString();
    }

    return paths;
  }
}
