/*
 * Copyright 2016 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package ru.ispras.retrascope.basis;

import org.junit.Test;
import ru.ispras.retrascope.Retrascope;

import java.util.Collection;

/**
 * Basic abstract class for tests that include sequential executions of the tool
 * on different command line arguments.
 * @author <a href="mailto:smolov@ispras.ru">Sergey Smolov</a>
 */
public abstract class RetrascopeExecTestCase {

  private final Collection<String[]> arguments;

  public RetrascopeExecTestCase(final Collection<String[]> arguments) {
    this.arguments = arguments;
  }

  private Collection<String[]> getArguments() {
    return this.arguments;
  }

  @Test
  public void runTest() {

    for (final String[] commandLine : getArguments()) {
      Retrascope.main(commandLine);
    }
  }
}
