/*
 * Copyright 2016-2019 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package ru.ispras.retrascope.engine.cfg.generator.test.random;

import org.junit.Test;
import ru.ispras.retrascope.HdlTestFiles;
import ru.ispras.retrascope.HdlTestUtils;
import ru.ispras.retrascope.Retrascope;
import ru.ispras.retrascope.engine.test.printer.testbench.vhdl.TestVhdlTestbenchPrinter;
import ru.ispras.retrascope.result.test.TestModel;
import ru.ispras.retrascope.result.testbench.VhdlTestbenchModel;

import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

/**
 * Test for random test generator based on Control Flow Graph Model.
 * The test runs test generator on every separate VHDL file and the engine produces
 * {@link TestModel} representation of test. After that the tool calls an engine of test
 * printing into VHDL format.
 *
 * @author <a href="mailto:smolov@ispras.ru">Sergey Smolov</a>
 */
public final class CfgRandomTestGeneratorVhdlTestCase extends CfgRandomTestGeneratorTestCase {

  @Test
  public void runTest() {
    final Map<String, Map<String, List<String>>> vhdlProjects = HdlTestFiles.VHDL_MAP;

    for (final Map.Entry<String, Map<String, List<String>>> prjEntry : vhdlProjects.entrySet()) {

      final Map<String, List<String>> prjFiles = prjEntry.getValue();

      for (final Map.Entry<String, List<String>> entry : prjFiles.entrySet()) {

        final String topName = entry.getKey();
        final List<String> filePaths = entry.getValue();

        final Map<String, List<String>> module = new LinkedHashMap<>();
        module.put(topName, filePaths);
        final ModuleDescriptor descriptor = getDescriptor(topName, filePaths);

        HdlTestUtils.runVhdl(
            module,
            VhdlTestbenchModel.TYPE.getId(),
            TestVhdlTestbenchPrinter.OVERWRITE.getName(),
            Retrascope.ENGINE_PARAM.getName(),
            CfgRandomTestGenerator.ENGINE_ID,
            CfgRandomTestGenerator.TEST_LEN.getName(),
            String.valueOf(TEST_LENGTH),
            CfgRandomTestGenerator.CLK_NAME.getName(),
            descriptor.getClkName(),
            CfgRandomTestGenerator.RST_NAME.getName(),
            descriptor.getRstName(),
            CfgRandomTestGenerator.INPUT_VALUES.getName(),
            descriptor.getInputValues());
      }
    }
  }
}
