/*
 * Copyright 2018-2019 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package ru.ispras.retrascope.engine.cfg.printer.graphml.sample;

import org.junit.Test;
import ru.ispras.fortress.expression.printer.ExprPrinter;
import ru.ispras.retrascope.Retrascope;
import ru.ispras.retrascope.VhdlTest;
import ru.ispras.retrascope.basis.Configuration;
import ru.ispras.retrascope.engine.cfg.printer.graphml.CfgGraphMlPrinter;
import ru.ispras.retrascope.util.LogLevel;

/**
 * Module-level test for CFG GraphML printer {@link CfgGraphMlPrinter}
 * that is applied to VHDL modules.
 *
 * @author <a href="mailto:smolov@ispras.ru">Sergey Smolov</a>
 */
public abstract class CfgGraphMlPrinterVhdlTest extends VhdlTest {

  /**
   * Returns the target module name.
   * @return The target module name.
   */
  protected abstract String getModuleName();

  @Test
  public void runTest() {

    addOption(Retrascope.TARGET_PARAM, CfgGraphMlPrinter.OUT_TYPE.getId());
    addOption(Configuration.PRINT_STYLE_PARAM, ExprPrinter.VHDL.name());
    addOption(Retrascope.LOG_LEVEL, LogLevel.DEBUG.getName());
    addOption(CfgGraphMlPrinter.OUT_FILE_NAME, String.format("%s-cfg.graphml", getModuleName()));

    super.runTest(getPaths());
  }
}
