/*
 * Copyright 2018-2019 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package ru.ispras.retrascope.engine.cfg.printer.graphml.sample;

import org.apache.commons.io.FilenameUtils;
import org.junit.Test;
import ru.ispras.retrascope.parser.verilog.VerilogParser;

import java.nio.file.Path;
import java.nio.file.Paths;

/**
 * The test runs the tool on "EX_stage.v" module
 * and generates GraphML representation for {@link ru.ispras.retrascope.model.cfg.CfgModel} model.
 *
 * @author <a href="mailto:smolov@ispras.ru">Sergey Smolov</a>
 */
public final class ExStageCfgGraphMlTestCase extends CfgGraphMlPrinterVerilogTest {

  @Override
  protected Path[] getModulePaths() {
    return new Path[] {
        Paths.get("opencores", "mips16", "EX_stage.v"),
        Paths.get("opencores", "mips16", "alu.v"),
    };
  }

  @Override
  protected String getModuleName() {
    return "EX_stage";
  }

  @Test
  public void runTest() {
    final String moduleDir = FilenameUtils.getFullPathNoEndSeparator(getPaths()[0]);
    addOption(VerilogParser.INCLUDE_PATH, moduleDir);
    super.runTest();
  }
}
