/*
 * Copyright 2018-2019 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package ru.ispras.retrascope.engine.cfg.transformer.gadd.sample;

import org.apache.commons.lang3.StringUtils;
import org.junit.Assert;
import org.junit.Test;
import ru.ispras.retrascope.Retrascope;
import ru.ispras.retrascope.VerilogTest;
import ru.ispras.retrascope.basis.exception.RetrascopeException;
import ru.ispras.retrascope.engine.cfg.transformer.gadd.CfgGaddTransformer;
import ru.ispras.retrascope.engine.cfg.transformer.gadd.backend.GaddPathCounterBackend;
import ru.ispras.retrascope.model.gadd.GaddModel;
import ru.ispras.retrascope.util.ReadWriteUtils;

import java.io.BufferedReader;
import java.io.DataInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;

/**
 * Module-level test for {@link CfgGaddTransformer} engine
 * that is applied to Verilog modules.
 *
 * @author <a href="mailto:smolov@ispras.ru">Sergey Smolov</a>
 */
public abstract class CfgGaddTransformerVerilogTest extends VerilogTest {

  abstract int getPathNum();

  @Test
  public void runTest() {

    addOption(Retrascope.TARGET_PARAM, GaddModel.TYPE.getId());
    addOption(Retrascope.ENGINE_PARAM, CfgGaddTransformer.ENGINE_ID);

    final String debugFileName = String.format("%s-gadd.log", getModuleName());
    addOption(Retrascope.TOOL_DEBUG_PARAM, debugFileName);

    super.runTest(getPaths());

    try {
      final FileInputStream inputStream = new FileInputStream(new File(debugFileName));
      final DataInputStream dataStream = new DataInputStream(inputStream);

      try (final BufferedReader reader = ReadWriteUtils.newBufferedReader(dataStream)) {

        String line;

        while ((line = reader.readLine()) != null) {

          if (!line.contains(GaddPathCounterBackend.DBG_PREFIX)) {
            continue;
          }
          final String pathNum = StringUtils.split(StringUtils.split(line)[1], '=')[1];

          final String refNum = Integer.toString(getPathNum());
          final String errMsg = String.format("[%s]: GADD path number error", getModuleName());

          Assert.assertEquals(errMsg, refNum, pathNum);
        }
      }
    } catch (final IOException e) {
      throw new RetrascopeException(e);
    }
  }
}
