/*
 * Copyright 2014-2019 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package ru.ispras.retrascope.engine.efsm.generator.test.fate;

import org.junit.Test;
import ru.ispras.retrascope.HdlTestUtils;
import ru.ispras.retrascope.Retrascope;
import ru.ispras.retrascope.result.test.TestModel;

import java.nio.file.Paths;
import java.util.Arrays;
import java.util.HashSet;
import java.util.Set;

/**
 * The test for {@link ru.ispras.retrascope.engine.efsm.generator.test.fate.EfsmFateTestGenerator}.
 * VHDL code examples are used as inputs.
 *
 * @author <al href="mailto:i.melnichenko@deltasolutions.ru">Igor Melnichenko</a>
 */
public class EfsmFateTestGeneratorVhdlTestCase {

  /*
  * Designs which cause failures at the moment.
  */
  private static final String[] IGNORED_PATHS = {
      Paths.get("b12", "b12.vhd").toString(),
      Paths.get("b13", "b13.vhd").toString(),
      Paths.get("b14", "b14.vhd").toString(),
      Paths.get("b14_1", "b14_1.vhd").toString(),
      Paths.get("b15", "b15.vhd").toString(),
      Paths.get("b15_1", "b15_1.vhd").toString(),
      Paths.get("b17", "b17.vhd").toString(),
      Paths.get("b17_1", "b17_1.vhd").toString(),
      Paths.get("b18", "b18.vhd").toString(),
      Paths.get("b18_1", "b18_1.vhd").toString(),
      Paths.get("b19", "b19.vhd").toString(),
      Paths.get("b19_1", "b19_1.vhd").toString(),
      Paths.get("b20", "b20.vhd").toString(),
      Paths.get("b20_1", "b20_1.vhd").toString(),
      Paths.get("b21", "b21.vhd").toString(),
      Paths.get("b21_1", "b21_1.vhd").toString(),
      Paths.get("b22", "b22.vhd").toString(),
      Paths.get("b22_1", "b22_1.vhd").toString(),
  };

  private static final Set<String> IGNORED = new HashSet<>(Arrays.asList(IGNORED_PATHS));

  @Test
  public void generate() {
    HdlTestUtils.runVhdl(TestModel.TYPE.getId(),
        IGNORED,
        Retrascope.ENGINE_PARAM.getName(),
        EfsmFateTestGenerator.ENGINE_ID,
            /*D_HYPHEN + Retrascope.LOG_PARAM.getLongName(),
            "fate.log",*/
        EfsmFateTestGenerator.LOOP_LIMIT.getName(),
        "25"//25 is for b11
            /*D_HYPHEN + EfsmFateTestGenerator.SEQUENCE_LENGTH_PARAMETER.getLongName(),
            "0",
            D_HYPHEN + EfsmFateTestGenerator.SEQUENCES_NUMBER_PARAMETER.getLongName(),
            "0",
            D_HYPHEN + Retrascope.LOG_LEVEL.getLongName(),
            "DEBUG"*/);
  }
}
