/*
 * Copyright 2018-2019 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package ru.ispras.retrascope.engine.efsm.printer.graphml.sample;

import org.apache.commons.io.FilenameUtils;
import org.junit.Test;
import ru.ispras.fortress.expression.printer.ExprPrinter;
import ru.ispras.retrascope.Retrascope;
import ru.ispras.retrascope.VerilogTest;
import ru.ispras.retrascope.basis.Configuration;
import ru.ispras.retrascope.engine.efsm.printer.graphml.EfsmGraphMlPrinter;
import ru.ispras.retrascope.parser.verilog.VerilogParser;

/**
 * Module-level test for EFSM GraphML printer {@link EfsmGraphMlPrinter}
 * that is applied to Verilog modules.
 *
 * @author <a href="mailto:smolov@ispras.ru">Sergey Smolov</a>
 */
public abstract class EfsmGraphMlPrinterVerilogTest extends VerilogTest {

  @Test
  public void runTest() {

    addOption(Retrascope.TARGET_PARAM, EfsmGraphMlPrinter.OUT_TYPE.getId());
    addOption(Configuration.PRINT_STYLE_PARAM, ExprPrinter.VERILOG.name());
    addOption(EfsmGraphMlPrinter.OUT_FILE_NAME, String.format("%s-gadd.graphml", getModuleName()));

    final String moduleDir = FilenameUtils.getFullPathNoEndSeparator(getPaths()[0]);
    addOption(VerilogParser.INCLUDE_PATH, moduleDir);

    super.runTest(getPaths());
  }
}
