/*
 * Copyright 2015-2019 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package ru.ispras.retrascope.engine.gadd.transformer.efsm;

import org.junit.Test;
import ru.ispras.retrascope.HdlTestFiles;
import ru.ispras.retrascope.Retrascope;
import ru.ispras.retrascope.model.efsm.EfsmModel;
import ru.ispras.retrascope.parser.basis.HdlParser;

import java.util.List;
import java.util.Map;

/**
 * Test that extracts EFSM model from b11 design with the initially specified state-like variables.
 *
 * @author <a href="mailto:smolov@ispras.ru">Sergey Smolov</a>
 */
public class B11GaddEfsmTransformerTestCase {

  private static final String ITC99_PROJECT = "itc99";
  private static final String B11 = "b11";
  private static final String B11_STATE_VARS = "STATO";

  @Test
  public void runTest() {
    final Map<String, Map<String, List<String>>> files = HdlTestFiles.VHDL_MAP;

    final Map<String, List<String>> project = files.get(ITC99_PROJECT);

    String b11path = null;
    for (final Map.Entry<String, List<String>> entry : project.entrySet()) {
      if (entry.getKey().equals(B11)) {
        b11path = entry.getValue().get(0);
      }
    }

    if (b11path != null) {

      Retrascope.main(
          Retrascope.TARGET_PARAM.getName(),
          EfsmModel.TYPE.getId(),
          Retrascope.ENGINE_PARAM.getName(),
          GaddEfsmTransformer.ENGINE_ID,
          Retrascope.TOOL_DEBUG_PARAM.getName(),
          "b11-efsm.log",
          b11path,
          HdlParser.MODULE_NAME.getName(),
          B11,
          GaddEfsmTransformer.STATE_VARIABLES.getName(),
          B11_STATE_VARS);
    }
  }
}
