/*
 * Copyright 2018 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package ru.ispras.retrascope.engine.gadd.transformer.efsm.sample;

/**
 * The container for {@link ru.ispras.retrascope.model.efsm.EfsmModel} model related
 * statistical information.
 *
 * @author <a href="mailto:smolov@ispras.ru">Sergey Smolov</a>
 */
public final class EfsmStatistics {

  private static final String STAT_FMT = "states=%d transitions=%d efsms=%d";

  private final int stateNum;
  private final int transNum;
  private final int modelNum;

  /**
   * Constructs the container for EFSM model related statistical information.
   * @param stateNum The total number of states.
   * @param transNum The total number of transitions.
   * @param modelNum The total number of EFSMs.
   */
  public EfsmStatistics(final int stateNum, final int transNum, final int modelNum) {
    this.stateNum = stateNum;
    this.transNum = transNum;
    this.modelNum = modelNum;
  }

  @Override
  public String toString() {
    return String.format(STAT_FMT, this.stateNum, this.transNum, this.modelNum);
  }
}
