/*
 * Copyright 2018-2019 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package ru.ispras.retrascope.engine.gadd.transformer.efsm.sample;

import org.apache.commons.lang3.StringUtils;
import org.junit.Assert;
import org.junit.Test;
import ru.ispras.retrascope.Retrascope;
import ru.ispras.retrascope.VhdlTest;
import ru.ispras.retrascope.basis.exception.RetrascopeException;
import ru.ispras.retrascope.engine.gadd.transformer.efsm.GaddEfsmTransformer;
import ru.ispras.retrascope.model.efsm.EfsmModel;
import ru.ispras.retrascope.util.ReadWriteUtils;

import java.io.BufferedReader;
import java.io.DataInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;

/**
 * Module-level test for {@link GaddEfsmTransformer} engine
 * that is applied to VHDL modules.
 *
 * @author <a href="mailto:smolov@ispras.ru">Sergey Smolov</a>
 */
public abstract class GaddEfsmTransformerVhdlTest extends VhdlTest {

  protected abstract EfsmStatistics getStatistics();

  @Test
  public void runTest() {

    final String debugFileName = String.format("%s-efsm.log", getModuleName());

    addOption(Retrascope.TARGET_PARAM, EfsmModel.TYPE.getId());
    addOption(Retrascope.ENGINE_PARAM, GaddEfsmTransformer.ENGINE_ID);
    addOption(Retrascope.TOOL_DEBUG_PARAM, debugFileName);

    super.runTest(getPaths());

    try {
      final FileInputStream inputStream = new FileInputStream(new File(debugFileName));
      final DataInputStream dataStream = new DataInputStream(inputStream);

      try (final BufferedReader reader = ReadWriteUtils.newBufferedReader(dataStream)) {

        String line;

        while ((line = reader.readLine()) != null) {

          if (line.length() == 0) {
            continue;
          }
          final String[] fragments = StringUtils.split(line);
          if (fragments.length < 2) {
            continue;
          }

          final String engineStr = fragments[0].toLowerCase();
          final String[] engineStrFragments = StringUtils.split(engineStr, '=');
          if (engineStrFragments.length < 2) {
            continue;
          }
          final String engine = engineStrFragments[1];

          if (!GaddEfsmTransformer.ENGINE_ID.equals(engine)) {
            continue;
          }

          final String prefixTopLevels = fragments[1].toLowerCase();
          final String[] splitByEq = StringUtils.split(prefixTopLevels, '=');

          /* NOTE: no-name model has "empty" name. */
          final String topLevelNames = splitByEq.length < 2 ? "empty" : splitByEq[1];

          final String refString = getStatistics().toString();
          final String wrongInfoMsg =
              String.format("[%s]: '%s' should end with '%s'", topLevelNames, line, refString);

          Assert.assertTrue(wrongInfoMsg, line.contains(refString));
        }
      }
    } catch (final IOException e) {
      throw new RetrascopeException(e);
    }
  }
}
