/*
 * Copyright 2019 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package ru.ispras.retrascope.engine.hldd.printer.smv.formula.sample;

import org.apache.commons.io.FilenameUtils;
import org.junit.Test;
import ru.ispras.retrascope.Retrascope;
import ru.ispras.retrascope.VerilogTest;
import ru.ispras.retrascope.engine.cfg.transformer.gadd.CfgGaddTransformer;
import ru.ispras.retrascope.engine.gadd.transformer.hldd.GaddHlddTransformer;
import ru.ispras.retrascope.engine.hldd.printer.smv.HlddSmvPrinter;
import ru.ispras.retrascope.engine.hldd.printer.smv.SmvTest;
import ru.ispras.retrascope.parser.basis.backend.CfgRangedSwitchBackend;
import ru.ispras.retrascope.parser.basis.backend.CfgVarRangeBlockBackend;
import ru.ispras.retrascope.parser.verilog.VerilogParser;

import java.io.File;

/**
 * Formula-based HLDD-to-SMV printer for Verilog designs.
 *
 * @author <a href="mailto:lebedev@ispras.ru">Mikhail Lebedev</a>
 */
public abstract class HlddSmvFormulaPrinterVerilogTest extends VerilogTest implements SmvTest {

  @Test
  public void runTest() {
    addOption(Retrascope.TARGET_PARAM, HlddSmvPrinter.OUT_TYPE.getId());
    addOption(Retrascope.ENGINE_PARAM, HlddSmvPrinter.ENGINE_ID);
    addOption(HlddSmvPrinter.SMV_FILE_NAME, getSmvFileName());
    addOption(GaddHlddTransformer.FORMULA_BASED);
    final String moduleDir = FilenameUtils.getFullPathNoEndSeparator(getPaths()[0]);
    addOption(VerilogParser.INCLUDE_PATH, moduleDir);
    addOption(CfgGaddTransformer.DISABLE_BACKENDS, String
        .format("%s%s%s", CfgVarRangeBlockBackend.NAME, File.pathSeparator,
            CfgRangedSwitchBackend.NAME));
    super.runTest(getPaths());
  }

}
