/*
 * Copyright 2015-2019 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package ru.ispras.retrascope.engine.hldd.printer.smv.property;

import org.junit.Assert;
import org.junit.Test;
import ru.ispras.retrascope.HdlTestFiles;
import ru.ispras.retrascope.HdlTestUtils;
import ru.ispras.retrascope.Retrascope;
import ru.ispras.retrascope.engine.gadd.transformer.efsm.GaddEfsmTransformer;
import ru.ispras.retrascope.util.ReadWriteUtils;

import java.io.File;
import java.util.Arrays;
import java.util.Collection;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;

/**
 * This class represents the test case for {@link HlddPropertySmvPrinter} class.
 *
 * @author <a href="mailto:lebedev@ispras.ru">Mikhail Lebedev</a>
 */
public final class HlddPropertySmvPrinterTestCase {

  /* SMV files that should not be created by the test. */
  private static final Collection<String> NO_MODEL_FILES =
      Arrays.asList("empty.smv", "mips_16_defs.smv");

  /* Names for modules that should be elaborated with user-defined state-like registers. */
  private static final Collection<String> STATE_DEF_MODULES = Arrays.asList("b07", "b08", "b11");

  @Test
  public void runTest() {

    final Map<String, Map<String, List<String>>> files = HdlTestFiles.getProjects();

    for (Entry<String, Map<String, List<String>>> project : files.entrySet()) {
      for (Entry<String, List<String>> file : project.getValue().entrySet()) {

        final String topName = file.getKey();
        final Map<String, List<String>> onlyFile = new LinkedHashMap<>();
        onlyFile.put(topName, file.getValue());
        final String smvFileName = topName + ".smv";

        ReadWriteUtils.clear(new File(smvFileName));

        if (STATE_DEF_MODULES.contains(topName)) {
          HdlTestUtils.runVhdl(
              onlyFile,
              HlddPropertySmvPrinter.OUT_TYPE.getId(),
              Retrascope.ENGINE_PARAM.getName(),
              HlddPropertySmvPrinter.ENGINE_ID,
              HlddPropertySmvPrinter.SMV_FILE_NAME.getName(),
              smvFileName,
              GaddEfsmTransformer.STATE_VARIABLES.getName(),
              "STATO");
        } else {
          HdlTestUtils.runHdl(
              onlyFile,
              HlddPropertySmvPrinter.OUT_TYPE.getId(),
              Retrascope.ENGINE_PARAM.getName(),
              HlddPropertySmvPrinter.ENGINE_ID,
              HlddPropertySmvPrinter.SMV_FILE_NAME.getName(),
              smvFileName);
        }

        if (!NO_MODEL_FILES.contains(smvFileName)) {

          Assert.assertTrue("Can't find file: " + smvFileName, ReadWriteUtils.exists(smvFileName));
        }
      }
    }
  }
}
