/*
 * Copyright 2015-2019 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package ru.ispras.retrascope.engine.smv;

import ru.ispras.retrascope.Retrascope;
import ru.ispras.retrascope.SmvUtilTest;
import ru.ispras.retrascope.basis.EntityType;
import ru.ispras.retrascope.engine.hldd.printer.smv.property.HlddPropertySmvPrinter;
import ru.ispras.retrascope.engine.smv.test.parser.SmvTestParser;
import ru.ispras.retrascope.engine.test.printer.testbench.TestbenchPrinterEngine;
import ru.ispras.retrascope.model.efsm.EfsmModel;
import ru.ispras.retrascope.model.gadd.GaddModel;

import java.io.File;
import java.util.List;

/**
 * Abstract class for {@link SmvTestParser} based test cases.
 *
 * @author <a href="mailto:lebedev@ispras.ru">Mikhail Lebedev</a>
 * @author <a href="mailto:smolov@ispras.ru">Sergey Smolov</a>
 */
abstract class SmvTestParserTest extends SmvUtilTest {

  @Override
  protected String[] getParameters(final String designName) {
    final List<String> targets = commonParameters(designName);

    targets.add(Retrascope.ENGINE_PARAM.getName());
    targets.add(String.format("%s%s%s", HlddPropertySmvPrinter.ENGINE_ID, File.pathSeparator,
        SmvTestParser.ENGINE_ID));
    targets.add(TestbenchPrinterEngine.OVERWRITE.getName());

    return targets.toArray(new String[targets.size()]);
  }

  @Override
  protected String getMainTarget() {
    return String
        .format("%s%s%s%s%s", getTargetType().getId(), File.pathSeparator, EfsmModel.TYPE.getId(),
            File.pathSeparator, GaddModel.TYPE.getId());
  }

  protected abstract EntityType getTargetType();
}
