/*
 * Copyright 2015-2016 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package ru.ispras.retrascope.model.cfg;

import org.junit.Assert;
import org.junit.Test;
import ru.ispras.fortress.expression.NodeValue;
import ru.ispras.fortress.expression.NodeVariable;
import ru.ispras.fortress.expression.Nodes;
import ru.ispras.retrascope.model.basis.DecStatement;
import ru.ispras.retrascope.model.basis.Declaration;
import ru.ispras.retrascope.model.basis.StatementUtils;

import java.util.Map;

/**
 * The test for {@link CfgUtils} class.
 *
 * @author <a href="mailto:smolov@ispras.ru">Sergey Smolov</a>
 */
public class CfgUtilsTestCase {

  @Test
  public void runTest() {

    runIsTypeTest();
    runGetVarContainerTest();
    runGetDeclaredTest();
  }

  private void runGetDeclaredTest() {

    final CfgModule module = new CfgModule("test_module");
    final NodeVariable xVariable = NodeVariable.newInteger("x");
    module.declareVariable(xVariable);

    final CfgProcess process = new CfgProcess();
    module.add(process);
    final NodeVariable yVariable = NodeVariable.newInteger("y");
    module.declareVariable(yVariable);
    final CfgBlockStatement blockStatement = new CfgBlockStatement(xVariable, yVariable);
    process.add(blockStatement);

    final Map<NodeVariable, Declaration> declared = CfgUtils.getDeclared(blockStatement);
    Assert.assertTrue(declared.size() == 2
        && declared.containsKey(xVariable) && declared.containsKey(yVariable));
  }

  private void runIsTypeTest() {

    CfgCaseStatement[] sameTypeNodes = new CfgCaseStatement[3];
    sameTypeNodes[0] = new CfgCaseStatement(NodeValue.newInteger(0));
    sameTypeNodes[1] = new CfgCaseStatement(NodeValue.newInteger(1));
    sameTypeNodes[2] = new CfgCaseStatement(NodeValue.newInteger(2));

    if (!StatementUtils.isType(CfgStatementType.CASE, sameTypeNodes)) {
      Assert.fail();
    }

    if (StatementUtils.isType(
        CfgStatementType.CASE,
        new CfgCaseStatement((NodeValue) Nodes.TRUE),
        new CfgSwitchStatement(Nodes.TRUE))) {
      Assert.fail();
    }
  }

  private void runGetVarContainerTest() {

    final CfgModule root = new CfgModule("root");
    final NodeVariable first = NodeVariable.newBoolean("first");
    root.declareVariable(first);

    final CfgProcess child = new CfgProcess();
    root.add(child);
    final NodeVariable second = NodeVariable.newBoolean("second");
    root.declareVariable(second);

    final NodeVariable third = NodeVariable.newBoolean("third");

    final DecStatement firstCon = StatementUtils.getVariableContainer(root, first.getName());
    if (!(firstCon != null && firstCon.equals(root))) {
      Assert.fail();
    }

    final DecStatement secondCon = StatementUtils.getVariableContainer(root, second.getName());
    if (!(secondCon != null && secondCon.equals(root))) {
      Assert.fail();
    }

    final DecStatement thirdCon = StatementUtils.getVariableContainer(root, third.getName());
    if (thirdCon != null) {
      Assert.fail();
    }
  }
}
