/*
 * Copyright 2017 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package ru.ispras.retrascope.model.efsm;

import org.junit.Assert;
import org.junit.Test;
import ru.ispras.fortress.expression.NodeVariable;
import ru.ispras.retrascope.model.basis.Module;
import ru.ispras.retrascope.model.basis.ModuleProcess;
import ru.ispras.retrascope.model.efsm.examples.B04;

import java.util.Collection;
import java.util.Set;

/**
 * Test for {@code Efsm} class utility methods that are applied to B04 model.
 * @author <a href="mailto:smolov@ispras.ru">Sergey Smolov</a>
 */
public class B04TestCase {

  @Test
  public void runTest() {
    final EfsmModel efsmModel = B04.EFSM_MODEL;
    final Collection<Module> modules = efsmModel.getModules();
    Assert.assertEquals(modules.size(), 1);

    final Module module = modules.iterator().next();
    final Collection<ModuleProcess> machines = module.getProcesses();
    Assert.assertEquals(machines.size(), 1);

    final ModuleProcess machine = machines.iterator().next();
    Assert.assertTrue(machine instanceof Efsm);

    final Efsm b04Efsm = (Efsm) machine;
    final Set<NodeVariable> stateVariables = b04Efsm.getStateVariables();
    Assert.assertEquals(stateVariables.size(), 1);

    final NodeVariable stateVariable = stateVariables.iterator().next();
    Assert.assertEquals(stateVariable.getName(), "stato");
  }
}
