/*
 * Copyright 2014-2018 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package ru.ispras.retrascope.model.efsm.examples;

import ru.ispras.fortress.expression.Node;
import ru.ispras.fortress.expression.NodeValue;
import ru.ispras.fortress.expression.NodeVariable;
import ru.ispras.fortress.expression.Nodes;
import ru.ispras.fortress.solver.constraint.Formulas;
import ru.ispras.retrascope.basis.MetaInfoType;
import ru.ispras.retrascope.basis.MetaInfoValue;
import ru.ispras.retrascope.model.basis.Assignment;
import ru.ispras.retrascope.model.basis.Event;
import ru.ispras.retrascope.model.basis.EventType;
import ru.ispras.retrascope.model.efsm.Action;
import ru.ispras.retrascope.model.efsm.Efsm;
import ru.ispras.retrascope.model.efsm.EfsmModel;
import ru.ispras.retrascope.model.efsm.EfsmModule;
import ru.ispras.retrascope.model.efsm.EfsmState;
import ru.ispras.retrascope.model.efsm.Guard;
import ru.ispras.retrascope.model.efsm.GuardedAction;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;

/**
 * The test class which contains a manually created ITC'99 b04 representation.
 *
 * @author <a href="mailto:i.melnichenko@deltasolutions.ru">Igor Melnichenko</a>
 */
public final class B04 {
  /**
   * An EFSM model which contains a manually created b04 EFSM representation.
   */
  public static final EfsmModel EFSM_MODEL = new EfsmModel();

  //TODO add transitions and reset guarded action

  private static final String BIT = "bit";

  static {
    final EfsmModule efsmModule = new EfsmModule("b04");

    final NodeVariable restart = ExampleEfsmUtils.declareInput(efsmModule, "RESTART", BIT);
    final NodeVariable average = ExampleEfsmUtils.declareInput(efsmModule, "AVERAGE", BIT);
    final NodeVariable enable = ExampleEfsmUtils.declareInput(efsmModule, "ENABLE", BIT);
    final NodeVariable dataIn =
        ExampleEfsmUtils.declareInput(efsmModule, "DATA_IN", "integer range 127 downto -128");

    final NodeVariable dataOut =
        ExampleEfsmUtils.declareOutput(efsmModule, "DATA_OUT", "integer range 127 downto -128");

    final NodeVariable clock = ExampleEfsmUtils.declareInput(efsmModule, "CLOCK", BIT);

    // TODO: uncomment when variable will be used somewhere
    /*NodeVariable reset = ExampleEfsmUtils.declareInput(efsmModule, "RESET", "bit");*/

    final Efsm efsm = new Efsm(efsmModule, "EFSM1");
    efsm.addMetaInfo(MetaInfoType.HDL_LANG, MetaInfoValue.LANG_VHDL);

    final NodeVariable stato =
        ExampleEfsmUtils.declareRegister(efsmModule, "stato", "integer range 2 downto 0");
    final NodeVariable rmax =
        ExampleEfsmUtils.declareRegister(efsmModule, "RMAX", "integer range 127 downto -128");
    final NodeVariable rmin =
        ExampleEfsmUtils.declareRegister(efsmModule, "RMIN", "integer range 127 downto -128");
    final NodeVariable rlast =
        ExampleEfsmUtils.declareRegister(efsmModule, "RLAST", "integer range 127 downto -128");
    final NodeVariable reg1 =
        ExampleEfsmUtils.declareRegister(efsmModule, "REG1", "integer range 127 downto -128");
    final NodeVariable reg2 =
        ExampleEfsmUtils.declareRegister(efsmModule, "REG2", "integer range 127 downto -128");
    final NodeVariable reg3 =
        ExampleEfsmUtils.declareRegister(efsmModule, "REG3", "integer range 127 downto -128");
    final NodeVariable reg4 =
        ExampleEfsmUtils.declareRegister(efsmModule, "REG4", "integer range 127 downto -128");
    final NodeVariable regd =
        ExampleEfsmUtils.declareRegister(efsmModule, "REGD", "integer range 127 downto -128");
    final NodeVariable temp = ExampleEfsmUtils.declareRegister(efsmModule, "temp", "integer");
    final NodeVariable res = ExampleEfsmUtils.declareRegister(efsmModule, "RES", BIT);
    final NodeVariable ave = ExampleEfsmUtils.declareRegister(efsmModule, "AVE", BIT);
    final NodeVariable ena = ExampleEfsmUtils.declareRegister(efsmModule, "ENA", BIT);

    final EfsmState stateA = efsm.addState(ExampleEfsmUtils.constructEquality(stato, "0"));
    final EfsmState stateB = efsm.addState(ExampleEfsmUtils.constructEquality(stato, "1"));
    final EfsmState stateC = efsm.addState(ExampleEfsmUtils.constructEquality(stato, "2"));

    efsm.setInitialState(stateC);

    final Node guardPredicate17 = Nodes.greater(dataIn, rmax);

    Formulas formulas = new Formulas();
    formulas.add(Nodes.less(dataIn, rmin));
    formulas.add(Nodes.lesseq(dataIn, rmax));
    final Node guardPredicate18 = formulas.asSingleExpr();

    final Node guardPredicate15 = Nodes.greatereq(Nodes.add(dataIn, reg4), NodeValue.newInteger(0));

    final Node guardPredicate16 = Nodes.less(Nodes.add(dataIn,reg4), NodeValue.newInteger(0));

    final Node guardPredicate6 = ExampleEfsmUtils.constructEquality(enable, "1");

    final Node guardPredicate7 = ExampleEfsmUtils.constructEquality(enable, "0");

    final Node guardPredicate9 = ExampleEfsmUtils.constructEquality(restart, "1");

    formulas = new Formulas();
    formulas.add(Nodes.noteq(restart, NodeValue.newBitVector(1, 1)));
    formulas.add(Nodes.noteq(enable, NodeValue.newBitVector(1, 1)));
    final Node guardPredicate8 = formulas.asSingleExpr();

    formulas = new Formulas();
    formulas.add(Nodes.noteq(restart, NodeValue.newBitVector(1, 1)));
    formulas.add(ExampleEfsmUtils.constructEquality(enable, "1"));
    final Node guardPredicate10 = formulas.asSingleExpr();

    final Node guardPredicate11 = ExampleEfsmUtils.constructEquality(average, "1");

    final Node guardPredicate12 = Nodes.noteq(average, NodeValue.newBitVector(1, 1));

    final Node guardPredicate13 = Nodes.greatereq(Nodes.add(rmax, rmin), NodeValue.newInteger(0));

    final Node guardPredicate14 = Nodes.less(Nodes.add(rmax, rmin), NodeValue.newInteger(0));

    final List<Assignment> statementsB = Arrays.asList(new Assignment(res, restart),
        new Assignment(ena, enable), new Assignment(ave, average));

    final List<Assignment> statementsD = Arrays.asList(new Assignment(rmax, dataIn),
        new Assignment(rmin, dataIn),
        new Assignment(reg1, ExampleEfsmUtils.constructValue(reg1, "0")),
        new Assignment(reg2, ExampleEfsmUtils.constructValue(reg2, "0")),
        new Assignment(reg3, ExampleEfsmUtils.constructValue(reg3, "0")),
        new Assignment(reg4, ExampleEfsmUtils.constructValue(reg4, "0")),
        new Assignment(rlast, ExampleEfsmUtils.constructValue(rlast, "0")),
        new Assignment(dataOut, ExampleEfsmUtils.constructValue(dataOut, "0")));

    final List<Assignment> statementsE = Collections.singletonList(new Assignment(rlast, dataIn));

    final List<Assignment> statementsG = new ArrayList<>(2);
    statementsG.add(new Assignment(regd,
        Nodes.mod(Nodes.add(rmax, rmin), NodeValue.newInteger(128))));
    statementsG.add(new Assignment(temp, Nodes.add(rmax, rmin)));

    final List<Assignment> statementsJ =
        Collections.singletonList(
            new Assignment(dataOut, Nodes.div(regd, NodeValue.newInteger(2))));

    final List<Assignment> statementsK =
        Collections.singletonList(
            new Assignment(dataOut,
                Nodes.minus(Nodes.div(Nodes.minus(regd), NodeValue.newInteger(2)))));

    final List<Assignment> statementsH = Collections.singletonList(new Assignment(dataOut, reg4));

    final List<Assignment> statementsI = new ArrayList<>(2);
    statementsI.add(
        new Assignment(regd, Nodes.mod(Nodes.add(dataIn, reg4), NodeValue.newInteger(128))));
    statementsI.add(new Assignment(temp, Nodes.add(dataIn, reg4)));

    final List<Assignment> statementsF = Collections.singletonList(new Assignment(dataOut, rlast));

    final List<Assignment> statementsN = Collections.singletonList(new Assignment(rmax, dataIn));

    final List<Assignment> statementsO = Collections.singletonList(new Assignment(rmin, dataIn));

    final List<Assignment> statementsP = Arrays.asList(new Assignment(reg4, reg3),
        new Assignment(reg3, reg2), new Assignment(reg2, reg1), new Assignment(reg1, dataIn));

    final Event clockEvent = new Event(clock, EventType.POSITIVE_EDGE);

    {
      final List<Node> guardPredicates1 = Arrays.asList(guardPredicate6, guardPredicate10,
          guardPredicate12, guardPredicate15, guardPredicate18);
      final List<Assignment> assignments1 = new ArrayList<>(statementsB);
      assignments1.addAll(statementsE);
      assignments1.addAll(statementsI);
      assignments1.addAll(statementsJ);
      assignments1.addAll(statementsO);
      assignments1.addAll(statementsP);

      final GuardedAction guardedAction1 =
          new GuardedAction(guardPredicates1, new Action(assignments1));
      guardedAction1.addClock(clockEvent);

      efsm.addTransition(stateC, stateC, guardedAction1);
    }

    {
      final List<Node> guardPredicates2 =
          Arrays.asList(guardPredicate7, guardPredicate8, guardPredicate17);
      final List<Assignment> assignments2 = new ArrayList<>(statementsB);
      assignments2.addAll(statementsF);
      assignments2.addAll(statementsN);
      assignments2.addAll(statementsP);

      final GuardedAction guardedAction2 =
          new GuardedAction(guardPredicates2, new Action(assignments2));
      guardedAction2.addClock(clockEvent);

      efsm.addTransition(stateC, stateC, guardedAction2);
    }

    {
      final List<Node> guardPredicates3 =
          Arrays.asList(guardPredicate7, guardPredicate9, guardPredicate13, guardPredicate17);
      final List<Assignment> assignments3 = new ArrayList<>(statementsB);
      assignments3.addAll(statementsG);
      assignments3.addAll(statementsJ);
      assignments3.addAll(statementsN);
      assignments3.addAll(statementsP);

      final GuardedAction guardedAction3 =
          new GuardedAction(guardPredicates3, new Action(assignments3));
      guardedAction3.addClock(clockEvent);

      efsm.addTransition(stateC, stateC, guardedAction3);
    }

    {
      final List<Node> guardPredicates4 =
          Arrays.asList(guardPredicate7, guardPredicate9, guardPredicate14, guardPredicate17);
      final List<Assignment> assignments4 = new ArrayList<>(statementsB);
      assignments4.addAll(statementsG);
      assignments4.addAll(statementsK);
      assignments4.addAll(statementsN);
      assignments4.addAll(statementsP);

      final GuardedAction guardedAction4 =
          new GuardedAction(guardPredicates4, new Action(assignments4));
      guardedAction4.addClock(clockEvent);

      efsm.addTransition(stateC, stateC, guardedAction4);
    }

    {
      final List<Node> guardPredicates8 =
          Arrays.asList(guardPredicate7, guardPredicate8, guardPredicate18);
      final List<Assignment> assignments8 = new ArrayList<>(statementsB);
      assignments8.addAll(statementsF);
      assignments8.addAll(statementsO);
      assignments8.addAll(statementsP);

      final GuardedAction guardedAction8 =
          new GuardedAction(guardPredicates8, new Action(assignments8));
      guardedAction8.addClock(clockEvent);

      efsm.addTransition(stateC, stateC, guardedAction8);
    }

    {
      final List<Node> guardPredicates9 =
          Arrays.asList(guardPredicate7, guardPredicate9, guardPredicate13, guardPredicate18);
      final List<Assignment> assignments9 = new ArrayList<>(statementsB);
      assignments9.addAll(statementsG);
      assignments9.addAll(statementsJ);
      assignments9.addAll(statementsO);
      assignments9.addAll(statementsP);
      final GuardedAction guardedAction9 =
          new GuardedAction(guardPredicates9, new Action(assignments9));
      guardedAction9.addClock(clockEvent);
      efsm.addTransition(stateC, stateC, guardedAction9);
    }

    {
      final List<Node> guardPredicates10 =
          Arrays.asList(guardPredicate7, guardPredicate9, guardPredicate14, guardPredicate18);
      final List<Assignment> assignments10 = new ArrayList<>(statementsB);
      assignments10.addAll(statementsG);
      assignments10.addAll(statementsK);
      assignments10.addAll(statementsO);
      assignments10.addAll(statementsP);
      final GuardedAction guardedAction10 =
          new GuardedAction(guardPredicates10, new Action(assignments10));
      guardedAction10.addClock(clockEvent);
      efsm.addTransition(stateC, stateC, guardedAction10);
    }

    {
      final GuardedAction guardedAction22 =
          new GuardedAction(ExampleEfsmUtils.ALWAYS_TRUE_GUARD, new Action(statementsB));
      guardedAction22.addClock(clockEvent);

      efsm.addTransition(stateA, stateB, guardedAction22);
    }

    {
      final List<Assignment> assignments23 = new ArrayList<>(statementsB);
      assignments23.addAll(statementsD);

      final GuardedAction guardedAction23 =
          new GuardedAction(ExampleEfsmUtils.ALWAYS_TRUE_GUARD, new Action(assignments23));
      guardedAction23.addClock(clockEvent);

      efsm.addTransition(stateB, stateC, guardedAction23);
    }

    {
      final List<Node> guardPredicates25 =
          Arrays.asList(guardPredicate6, guardPredicate9, guardPredicate13, guardPredicate17);
      final List<Assignment> assignments25 = new ArrayList<>(statementsB);
      assignments25.addAll(statementsE);
      assignments25.addAll(statementsG);
      assignments25.addAll(statementsJ);
      assignments25.addAll(statementsN);
      assignments25.addAll(statementsP);
      final GuardedAction guardedAction25 =
          new GuardedAction(guardPredicates25, new Action(assignments25));
      guardedAction25.addClock(clockEvent);
      efsm.addTransition(stateC, stateC, guardedAction25);
    }

    {
      final List<Node> guardPredicates26 =
          Arrays.asList(guardPredicate6, guardPredicate9, guardPredicate14, guardPredicate17);
      final List<Assignment> assignments26 = new ArrayList<>(statementsB);
      assignments26.addAll(statementsE);
      assignments26.addAll(statementsG);
      assignments26.addAll(statementsK);
      assignments26.addAll(statementsN);
      assignments26.addAll(statementsP);
      final GuardedAction guardedAction26 =
          new GuardedAction(guardPredicates26, new Action(assignments26));
      guardedAction26.addClock(clockEvent);
      efsm.addTransition(stateC, stateC, guardedAction26);
    }

    {
      final Guard guard27 =
          new Guard(
              Arrays.asList(guardPredicate6, guardPredicate10, guardPredicate11, guardPredicate17));
      final List<Assignment> assignments27 = new ArrayList<>(statementsB);
      assignments27.addAll(statementsE);
      assignments27.addAll(statementsH);
      assignments27.addAll(statementsN);
      assignments27.addAll(statementsP);

      final GuardedAction guardedAction27 = new GuardedAction(guard27, new Action(assignments27));
      guardedAction27.addClock(clockEvent);

      efsm.addTransition(stateC, stateC, guardedAction27);
    }

    {
      final List<Node> guardPredicates28 =
          Arrays.asList(
              guardPredicate6,
              guardPredicate10,
              guardPredicate12,
              guardPredicate15,
              guardPredicate17);
      final List<Assignment> assignments28 = new ArrayList<>(statementsB);
      assignments28.addAll(statementsE);
      assignments28.addAll(statementsI);
      assignments28.addAll(statementsJ);
      assignments28.addAll(statementsN);
      assignments28.addAll(statementsP);

      final GuardedAction guardedAction28 =
          new GuardedAction(guardPredicates28, new Action(assignments28));
      guardedAction28.addClock(clockEvent);

      efsm.addTransition(stateC, stateC, guardedAction28);
    }

    {
      final List<Node> guardPredicates29 =
          Arrays.asList(
              guardPredicate6,
              guardPredicate10,
              guardPredicate12,
              guardPredicate16,
              guardPredicate17);
      final List<Assignment> assignments29 = new ArrayList<>(statementsB);
      assignments29.addAll(statementsE);
      assignments29.addAll(statementsI);
      assignments29.addAll(statementsK);
      assignments29.addAll(statementsN);
      assignments29.addAll(statementsP);

      final GuardedAction guardedAction29 =
          new GuardedAction(guardPredicates29, new Action(assignments29));
      guardedAction29.addClock(clockEvent);

      efsm.addTransition(stateC, stateC, guardedAction29);
    }

    {
      final List<Node> guardPredicates31 =
          Arrays.asList(guardPredicate6, guardPredicate9, guardPredicate13, guardPredicate18);
      final List<Assignment> assignments31 = new ArrayList<>(statementsB);
      assignments31.addAll(statementsE);
      assignments31.addAll(statementsG);
      assignments31.addAll(statementsJ);
      assignments31.addAll(statementsO);
      assignments31.addAll(statementsP);

      final GuardedAction guardedAction31 =
          new GuardedAction(guardPredicates31, new Action(assignments31));
      guardedAction31.addClock(clockEvent);

      efsm.addTransition(stateC, stateC, guardedAction31);
    }

    {
      final List<Node> guardPredicates32 =
          Arrays.asList(guardPredicate6, guardPredicate9, guardPredicate14, guardPredicate18);
      final List<Assignment> assignments32 = new ArrayList<>(statementsB);
      assignments32.addAll(statementsE);
      assignments32.addAll(statementsG);
      assignments32.addAll(statementsK);
      assignments32.addAll(statementsO);
      assignments32.addAll(statementsP);

      final GuardedAction guardedAction32 =
          new GuardedAction(guardPredicates32, new Action(assignments32));
      guardedAction32.addClock(clockEvent);

      efsm.addTransition(stateC, stateC, guardedAction32);
    }

    {
      final List<Node> guardPredicates33 =
          Arrays.asList(guardPredicate6, guardPredicate10, guardPredicate11, guardPredicate18);
      final List<Assignment> assignments33 = new ArrayList<>(statementsB);
      assignments33.addAll(statementsE);
      assignments33.addAll(statementsH);
      assignments33.addAll(statementsO);
      assignments33.addAll(statementsP);

      final GuardedAction guardedAction33 =
          new GuardedAction(guardPredicates33, new Action(assignments33));
      guardedAction33.addClock(clockEvent);

      efsm.addTransition(stateC, stateC, guardedAction33);
    }

    // TODO: uncomment when event list will be used somewhere
    /*Event resetEvent = new Event(new RangedVariable(reset), EventType.POSITIVE_EDGE);
    ArrayList<Event> resetEventList = new ArrayList<>(1);
    resetEventList.add(resetEvent);*/

    efsmModule.addNonInitialEfsm(efsm);
    B04.EFSM_MODEL.addModule(efsmModule);
  }

  private B04() {
    /* Empty. */
  }
}
