/*
 * Copyright 2019 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package ru.ispras.retrascope.parser;

/**
 * The container for CFG model related statistical information.
 *
 * @author <a href="mailto:smolov@ispras.ru">Sergey Smolov</a>
 */
public final class CfgStatistics {

  private static final String FORMAT = "modules=%d variables=%d processes=%d switches=%d blocks=%d";

  private final int moduleNum;
  private final int varNum;
  private final int processNum;
  private final int switchNum;
  private final int blockNum;

  /**
   * Constructs the container for statistical information.
   * @param moduleNum The total number of modules.
   * @param varNum The total number of variables.
   * @param processNum The total number of processes.
   * @param switchNum The total number of switch statements.
   * @param blockNum The total number of basic statements.
   */
  public CfgStatistics(
      final int moduleNum,
      final int varNum,
      final int processNum,
      final int switchNum,
      final int blockNum) {
    this.moduleNum = moduleNum;
    this.varNum = varNum;
    this.processNum = processNum;
    this.switchNum = switchNum;
    this.blockNum = blockNum;
  }

  @Override
  public String toString() {

    return String.format(FORMAT,
        this.moduleNum,
        this.varNum,
        this.processNum,
        this.switchNum,
        this.blockNum);
  }
}
