/*
 * Copyright 2015-2019 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package ru.ispras.retrascope.parser.vhdl;

import ru.ispras.retrascope.HdlTestFiles;
import ru.ispras.retrascope.Retrascope;
import ru.ispras.retrascope.basis.RetrascopeExecTestCase;
import ru.ispras.retrascope.model.cfg.CfgModel;

import java.util.Collection;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

/**
 * This test runs the tool on every single VHDL module without specifying the "toplevel" option.
 * @author <a href="mailto:smolov@ispras.ru">Sergey Smolov</a>
 */
public class VhdlParserModuleNoTopLevelTestCase extends RetrascopeExecTestCase {

  private static final Collection<String[]> CMD_LINES = new LinkedList<>();

  static {
    final Map<String, Map<String, List<String>>> files = HdlTestFiles.VHDL_MAP;

    for (final Map.Entry<String, Map<String, List<String>>> project : files.entrySet()) {
      for (final List<String> filePaths : project.getValue().values()) {
        for (final String filePath : filePaths) {

          if (filePath.endsWith("b30.vhd")) {

            /* For this design the tool MUST use toplevel option, so it is skipped. */
            continue;
          }
          CMD_LINES.add(new String[] {
              Retrascope.TARGET_PARAM.getName(),
              CfgModel.TYPE.getId(),
              filePath});
        }
      }
    }
  }

  /**
   * Default constructor.
   */
  public VhdlParserModuleNoTopLevelTestCase() {
    super(CMD_LINES);
  }
}
