/*
 * Copyright 2014-2019 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package ru.ispras.retrascope.parser.vhdl;

import org.junit.Assert;
import org.junit.Test;
import ru.ispras.retrascope.Retrascope;
import ru.ispras.retrascope.VhdlTest;
import ru.ispras.retrascope.basis.exception.RetrascopeException;
import ru.ispras.retrascope.model.cfg.CfgModel;
import ru.ispras.retrascope.parser.CfgStatistics;
import ru.ispras.retrascope.parser.basis.HdlParser;
import ru.ispras.retrascope.util.LogLevel;
import ru.ispras.retrascope.util.ReadWriteUtils;

import java.io.BufferedReader;
import java.io.DataInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;

/**
 * Module-level test for VHDL parser {@link VhdlParser}.
 *
 * @author <a href="mailto:smolov@ispras.ru">Sergey Smolov</a>
 */
public abstract class VhdlParserTest extends VhdlTest {

  /**
   * Returns the "module under test" name.
   * @return The name of module to be tested.
   */
  protected abstract String getModuleName();

  protected abstract CfgStatistics getStatistics();

  @Test
  public void runTest() {

    addOption(Retrascope.TARGET_PARAM, CfgModel.TYPE.getId());
    addOption(Retrascope.ENGINE_PARAM, VhdlParser.ENGINE_ID);
    addOption(HdlParser.MODULE_NAME, getModuleName());
    addOption(Retrascope.LOG_LEVEL, LogLevel.DEBUG.getName());

    final String debugFileName = String.format("%s-cfg.log", getModuleName());
    addOption(Retrascope.TOOL_DEBUG_PARAM, debugFileName);

    super.runTest(getPaths());

    try {
      final FileInputStream inputStream = new FileInputStream(new File(debugFileName));
      final DataInputStream dataStream = new DataInputStream(inputStream);

      try (final BufferedReader reader = ReadWriteUtils.newBufferedReader(dataStream)) {

        String line;

        while ((line = reader.readLine()) != null) {

          if (!line.contains("hdl-parser")) {
            continue;
          }

          final String refString = getStatistics().toString();
          final String errMsg = String.format("'%s' should contain '%s'", line, refString);

          Assert.assertTrue(errMsg, line.endsWith(refString));
        }
      }
    } catch (final IOException e) {
      throw new RetrascopeException(e);
    }
  }
}
