/*
 * Copyright 2015-2018 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package ru.ispras.retrascope.result.property;

import org.junit.Assert;
import org.junit.Test;
import ru.ispras.fortress.expression.Node;
import ru.ispras.fortress.expression.NodeValue;
import ru.ispras.fortress.expression.NodeVariable;
import ru.ispras.fortress.expression.Nodes;
import ru.ispras.retrascope.model.basis.Assignment;
import ru.ispras.retrascope.model.efsm.Action;
import ru.ispras.retrascope.model.efsm.GuardedAction;

import java.util.Collections;
import java.util.List;

/**
 * Test for assertion result's inner representation.
 *
 * @see PropertyEntity
 * @see ModuleProperty
 * @see ProcessProperty
 *
 * @author <a href="mailto:smolov@ispras.ru">Sergey Smolov</a>
 */
public class PropertyEntityTestCase {

  @Test
  public void runTest() {

    /* Create top-level entity. */
    final PropertyEntity entity = new PropertyEntity();

    /* Create module-level containers. */
    final ModuleProperty container1 = new ModuleProperty("1");
    entity.addContainer(container1);

    final ModuleProperty container2 = new ModuleProperty("2");
    entity.addContainer(container2);

    final ModuleProperty container3 = new ModuleProperty("3", container2);
    container2.addModuleProperty(container3);

    /* Create process-level groups. */
    final Node initState = Nodes.TRUE;
    final GuardedAction rstGuardedAction =
        new GuardedAction(
            initState,
            new Action(new Assignment(NodeVariable.newInteger("x"), NodeValue.newInteger(0))));

    final List<GuardedAction> rstGuardedActions = Collections.singletonList(rstGuardedAction);
    final ProcessProperty group1 =
        new ProcessProperty("1", container1, initState, rstGuardedActions);
    container1.addProcessProperty(group1);
    Assert.assertEquals(container1.getProcessMap().size(), 1);

    final ProcessProperty group2 =
        new ProcessProperty("2", container2, initState, rstGuardedActions);
    container2.addProcessProperty(group2);

    final ProcessProperty group3 =
        new ProcessProperty("3", container3, initState, rstGuardedActions);
    container3.addProcessProperty(group3);

    /* Variables. */
    final NodeVariable varOne = NodeVariable.newInteger("one");
    final NodeVariable varTwo = NodeVariable.newInteger("two");
    final NodeVariable varThree = NodeVariable.newInteger("three");

    /* Declare variables. */
    container1.declareVariable(varOne);
    container2.declareVariable(varTwo);
    container3.declareVariable(varThree);

    /* Assertions. */
    final Node equationX = Nodes.eq(varOne, NodeValue.newInteger(0));
    final Node equationY = Nodes.eq(varTwo, NodeValue.newInteger(0));
    final Node equationZ = Nodes.eq(varThree, NodeValue.newInteger(0));

    /* Try to add properties. */
    group1.addProperty(equationX);
    group2.addProperty(equationY);
    group3.addProperty(equationZ);
  }
}
