/*
 * Copyright 2014-2016 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package ru.ispras.retrascope.basis;

import org.apache.commons.cli.ParseException;
import ru.ispras.fortress.expression.printer.ExprPrinter;
import ru.ispras.fortress.expression.printer.ExprTreePrinter;
import ru.ispras.fortress.util.InvariantChecks;
import ru.ispras.retrascope.util.Log;
import ru.ispras.retrascope.util.LogMessage;

import java.util.Arrays;

/**
 * This class implements tool {@link ru.ispras.retrascope.Retrascope} configuration.
 * <p>
 * It contains predefined tool parameters and their handlers.
 * </p>
 * @author <a href="mailto:smolov@ispras.ru">Sergey Smolov</a>
 */
public class Configuration {

  /** The predefined parameters. */
  public static final Parameter PRINT_STYLE_PARAM =
      new Parameter("printer-style", ExprPrinter.names(), "Set a printer style");

  /** The parameters handlers. */
  private ExprTreePrinter stylePrinter;

  /** The configuration parameters. */
  public static final Parameters PARAMETERS = new Parameters();

  static {
    PARAMETERS.addParameter(PRINT_STYLE_PARAM);
  }

  private void setStylePrinter(final ExprTreePrinter printer) {
    InvariantChecks.checkNotNull(printer);
    this.stylePrinter = printer;
  }

  /**
   * Configures this object using command line arguments.
   *
   * @param args command line
   */
  public void configure(final String ... args) {

    final Arguments arguments = new Arguments(PARAMETERS);

    try {
      arguments.setCommandLine(args);
    } catch (ParseException e) {
      Log.error("%s%s.", LogMessage.WRONG_ARG, Arrays.toString(args));
    }

    setStylePrinter(ExprPrinter.getExprPrinter(
        arguments.getValue( PRINT_STYLE_PARAM, ExprPrinter.VERILOG.name())));
  }

  /**
   * Returns expression style printer.
   *
   * @return expression printer
   */
  public ExprTreePrinter getStylePrinter() {
    return this.stylePrinter;
  }
}
