/*
 * Copyright 2018 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package ru.ispras.retrascope.engine.basis;

import ru.ispras.retrascope.basis.EntityType;
import ru.ispras.retrascope.basis.Parameter;
import ru.ispras.retrascope.model.basis.walker.ModelWalker;

/**
 * Abstract GraphML printer engine.
 * @author <a href="mailto:smolov@ispras.ru">Sergey Smolov</a>
 */
public abstract class GraphMlPrinterEngine extends PrinterWalkerEngine {

  /**
   * The parameter that keeps the name for the generated GraphML file.
   */
  public static final Parameter OUT_FILE_NAME =
      new Parameter("graphml", true, "Output GraphML file name");

  /**
   * Constructs object with the specified identifier, input/output types,
   * walkers.
   *
   * @param engineId   Engine identifier.
   * @param inputType  Engine model input type.
   * @param outputType Engine model output type.
   * @param walkers    Model walkers.
   * @throws IllegalArgumentException when walkers array is {@code null}.
   */
  protected GraphMlPrinterEngine(
      final String engineId,
      final EntityType inputType,
      final EntityType outputType,
      final ModelWalker... walkers) {
    super(engineId, inputType, outputType, walkers);
    addParameter(OUT_FILE_NAME);
  }

  @Override
  protected String getOutFileExt() {
    return "graphml";
  }

  @Override
  protected Parameter getOutFileName() {
    return OUT_FILE_NAME;
  }
}
