/*
 * Copyright 2018 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package ru.ispras.retrascope.engine.basis;

import org.apache.commons.cli.ParseException;
import ru.ispras.retrascope.basis.Arguments;
import ru.ispras.retrascope.basis.Entity;
import ru.ispras.retrascope.basis.EntityType;
import ru.ispras.retrascope.basis.FileNames;
import ru.ispras.retrascope.basis.Parameter;
import ru.ispras.retrascope.basis.exception.RetrascopeException;
import ru.ispras.retrascope.model.basis.walker.ModelWalker;

import java.util.Map;

/**
 * Abstract engine that walks model and prints the result in some format.
 *
 * @author <a href="mailto:smolov@ispras.ru">Sergey Smolov</a>
 */
public abstract class PrinterWalkerEngine extends ModelWalkerEngine implements PrinterEngine {

  // The printer result.
  private FileNames fileNames;

  /**
   * Constructs object with the specified identifier, input/output types,
   * walkers.
   *
   * @param engineId   Engine identifier.
   * @param inputType  Engine model input type.
   * @param outputType Engine model output type.
   * @param walkers    Model walkers.
   * @throws IllegalArgumentException when walkers array is {@code null}.
   */
  protected PrinterWalkerEngine(
      final String engineId,
      final EntityType inputType,
      final EntityType outputType,
      final ModelWalker... walkers) {
    super(engineId, inputType, outputType, walkers);
  }

  protected abstract Parameter getOutFileName();

  /**
   * Returns the extension for generated files.
   *
   * @return The string that is an extension for the output file.
   */
  protected abstract String getOutFileExt();

  /**
   * Initializes visitor with the spcified file name.
   *
   * @param fileName The string name for output file.
   */
  protected abstract void initVisitor(final String fileName);

  @Override
  public void initialize(final Map<EntityType, Entity> inputs) {

    final Arguments arguments = (Arguments)inputs.get(Arguments.TYPE);

    try {

      final Arguments engineArgs = parseCommandLine(arguments.getCommandLine());

      this.fileNames = new FileNames(getOutFileExt());

      final String fileNameValue = engineArgs.getValue(getOutFileName(), getDefaultFileName());

      fileNames.add(fileNameValue);

      initVisitor(fileNameValue);

    } catch (ParseException e) {
      throw new RetrascopeException(e);
    }
  }

  @Override
  public Entity getOutput() {
    return this.fileNames;
  }
}
