/*
 * Copyright 2018 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package ru.ispras.retrascope.engine.basis;

import ru.ispras.retrascope.model.basis.memory.HashMapMemory;

/**
 * The snapshot of simulation.
 *
 * @author <a href="mailto:i.melnichenko@deltasolutions.ru">Igor Melnichenko</a>
 * @author <a href="mailto:smolov@ispras.ru">Sergey Smolov</a>
 *
 * @see Simulator
 */
public class SimulationSnapshot {

  private final HashMapMemory inMemory;
  private final HashMapMemory outMemory;
  private final HashMapMemory regMemory;

  protected SimulationSnapshot(final Simulated simulator) {
    this.inMemory = new HashMapMemory(simulator.getInputsMemory());
    this.regMemory = new HashMapMemory(simulator.getRegMemory());
    this.outMemory = new HashMapMemory(simulator.getOutputsMemory());
  }

  public HashMapMemory getInMemory() {
    return inMemory;
  }

  public HashMapMemory getOutMemory() {
    return outMemory;
  }

  public HashMapMemory getRegMemory() {
    return regMemory;
  }
}
