/*
 * Copyright 2018-2019 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package ru.ispras.retrascope.engine.cfg.printer.c;

import ru.ispras.retrascope.basis.EntityType;
import ru.ispras.retrascope.basis.Parameter;
import ru.ispras.retrascope.engine.basis.PrinterWalkerEngine;
import ru.ispras.retrascope.model.basis.walker.WalkerMode;
import ru.ispras.retrascope.model.cfg.CfgDfsWalker;
import ru.ispras.retrascope.model.cfg.CfgModel;

/**
 * Printer for {@link CfgModel} model to C programming language format.
 *
 * @author <a href="mailto:smolov@ispras.ru">Sergey Smolov</a>
 */
public class CfgCPrinter extends PrinterWalkerEngine {

  /**
   * The engine identifier.
   */
  public static final String ENGINE_ID = "cfg-c-printer";

  /**
   * The engine output entity type.
   */
  public static final EntityType OUT_TYPE = EntityType.get("cfg-c");

  /**
   * The parameter for output C file name.
   */
  public static final Parameter OUT_FILE_NAME = new Parameter("c", true, "Output C file name.");

  private static final String TEMPLATE = "stg/c/CfgModel.stg";

  /**
   * Default constructor.
   */
  public CfgCPrinter() {
    super(
        ENGINE_ID,
        CfgModel.TYPE,
        OUT_TYPE,
        new CfgDfsWalker(new CfgCVisitor(), WalkerMode.DFS_NO_RPT));

    addParameter(OUT_FILE_NAME);
  }

  @Override
  protected Parameter getOutFileName() {
    return OUT_FILE_NAME;
  }

  @Override
  protected String getOutFileExt() {
    return "c";
  }

  @Override
  protected void initVisitor(final String fileName) {
    ((CfgCVisitor)getInitVisitor()).initialize(fileName, TEMPLATE);
  }

  @Override
  public String getDefaultFileName() {
    return "cfg-model.c";
  }
}
