/*
 * Copyright 2014-2018 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package ru.ispras.retrascope.engine.cfg.printer.graphml;

import ru.ispras.retrascope.basis.EntityType;
import ru.ispras.retrascope.engine.basis.GraphMlPrinterEngine;
import ru.ispras.retrascope.model.basis.walker.ModelWalker;
import ru.ispras.retrascope.model.basis.walker.WalkerMode;
import ru.ispras.retrascope.model.cfg.CfgDfsWalker;
import ru.ispras.retrascope.model.cfg.CfgModel;

/**
 * Control flow graph printer to GraphML format.
 *
 * @author <a href="mailto:protsenko@ispras.ru">Alexander Protsenko</a>
 * @author <a href="mailto:smolov@ispras.ru">Sergey Smolov</a>
 */

public class CfgGraphMlPrinter extends GraphMlPrinterEngine {

  /**
   * The engine identifier.
   */
  public static final String ENGINE_ID = "cfg-graphml-printer";

  /**
   * The engine output entity type.
   */
  public static final EntityType OUT_TYPE = EntityType.get("cfg-graphml");

  /**
   * Default constructor.
   */
  public CfgGraphMlPrinter() {
    super(
        ENGINE_ID,
        CfgModel.TYPE,
        OUT_TYPE,
        new CfgDfsWalker(new CfgGraphMlVisitor(), WalkerMode.DFS_NO_RPT));

    for (final ModelWalker walker : getWalkers()) {
      final CfgGraphMlVisitor visitor = (CfgGraphMlVisitor) walker.getVisitor();
      visitor.setWalker(walker);
    }
  }

  /**
   * Initialize visitor with the graphML file name.
   *
   * @param fileName Output file name.
   */
  public void initVisitor(final String fileName) {
    ((CfgGraphMlVisitor) getInitVisitor()).initialize(fileName);
  }


  /**
   * Print the control flow graph to GraphML file.
   *
   * @param fileName Output file name.
   * @param cfg Control flow graph to print.
   */
  public void print(final String fileName, final CfgModel cfg) {

    // Create an output file for the GraphML printer.
    final CfgGraphMlVisitor cfgGraphMlFile = new CfgGraphMlVisitor();
    cfgGraphMlFile.initialize(fileName);

    // Construct a CFG walker for the GraphML printer.
    final ModelWalker cfgWalker = new CfgDfsWalker(cfgGraphMlFile, WalkerMode.DFS_NO_RPT);
    cfgGraphMlFile.setWalker(cfgWalker);

    // Run the CFG walker.
    cfgWalker.start(cfg);
  }

  @Override
  public String getDefaultFileName() {
    return "cfg-model.graphml";
  }
}
