/*
 * Copyright 2017-2019 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package ru.ispras.retrascope.engine.cfg.transformer.gadd;

import ru.ispras.retrascope.model.basis.Module;
import ru.ispras.retrascope.model.basis.Statement;
import ru.ispras.retrascope.model.basis.walker.ModelWalker;
import ru.ispras.retrascope.model.basis.walker.WalkerMode;
import ru.ispras.retrascope.model.cfg.CfgDfsWalker;

/**
 * Utility methods for control flow graph model transformation.
 * @author <a href="mailto:smolov@ispras.ru">Sergey Smolov</a>
 */
final class CfgSsaTransformer {

  /* The mode for walkers to build SSA form. */
  private static final WalkerMode MODE = WalkerMode.DFS_NO_RPT;

  private CfgSsaTransformer() {
    /* Empty. */
  }

  /**
   * Transforms the specified statement to a Static Single Assignment (SSA) form.
   *
   * @param top The top level module.
   * @param statement The statement to be transformed.
   */
  static void toSsa(final Module top, final Statement statement) {

    /* Transform statement to SSA form. */
    final ModelWalker ssaBlockWalker = new CfgDfsWalker(new CfgSsaBlockVisitor(), MODE);
    ssaBlockWalker.visitNode(statement);

    final ModelWalker ssaWalker = new CfgDfsWalker(new CfgSsaVersionVisitor(top), MODE);
    ssaWalker.visitNode(statement);
  }
}
