/*
 * Copyright 2019 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package ru.ispras.retrascope.engine.cfg.transformer.gadd.backend;

import ru.ispras.retrascope.basis.Entity;
import ru.ispras.retrascope.engine.cfg.CfgVisitorBackend;
import ru.ispras.retrascope.engine.cfg.transformer.gadd.CfgGaddTransformer;
import ru.ispras.retrascope.model.basis.walker.WalkerMode;
import ru.ispras.retrascope.util.Log;

/**
 * Transformer component that calculates the number of model paths.
 *
 * @author <a href="mailto:smolov@ispras.ru">Sergey Smolov</a>
 */
public class GaddPathCounterBackend extends CfgVisitorBackend {

  /**
   * The backend name.
   */
  public static final String NAME = "path_counter";

  /**
   * The beginning of backend's debug writes.
   */
  public static final String DBG_PREFIX = "engine=" + CfgGaddTransformer.ENGINE_ID;

  public GaddPathCounterBackend() {
    super(GaddPathCounterBackend.NAME, new PathCountVisitor(), WalkerMode.DFS_NO_RPT);
  }

  @Override
  public void start(final Entity entity) {

    ((PathCountVisitor)getVisitor()).setPathNum(0);
    super.start(entity);

    final int pathNum = ((PathCountVisitor) getVisitor()).getPathNum();
    final String statistics = String.format("%s paths=%d\n", DBG_PREFIX, pathNum);

    Log.debug(statistics);
    debugWrite(statistics);
  }
}
