/*
 * Copyright 2014-2018 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package ru.ispras.retrascope.engine.efsm.printer.graphml;

import ru.ispras.retrascope.basis.EntityType;
import ru.ispras.retrascope.engine.basis.GraphMlPrinterEngine;
import ru.ispras.retrascope.model.basis.walker.ModelWalker;
import ru.ispras.retrascope.model.efsm.EfsmDfsWalker;
import ru.ispras.retrascope.model.efsm.EfsmModel;

/**
 * EFSM printer to GraphML format.
 *
 * @author <a href="mailto:protsenko@ispras.ru">Alexander Protsenko</a>
 * @author <a href="mailto:smolov@ispras.ru">Sergey Smolov</a>
 */

public class EfsmGraphMlPrinter extends GraphMlPrinterEngine {

  /**
   * The engine identifier.
   */
  public static final String ENGINE_ID = "efsm-graphml-printer";

  /**
   *  The engine output entity type.
   */
  public static final EntityType OUT_TYPE = EntityType.get("efsm-graphml");

  /**
   * The default constructor.
   */
  public EfsmGraphMlPrinter() {
    super(
        EfsmGraphMlPrinter.ENGINE_ID,
        EfsmModel.TYPE,
        EfsmGraphMlPrinter.OUT_TYPE,
        new EfsmDfsWalker(new EfsmGraphMlVisitor()));
  }

  @Override
  protected void initVisitor(final String fileName) {
    ((EfsmGraphMlVisitor)getWalkers()[0].getVisitor()).initFile(fileName);
  }

  /**
   * Prints the specified {@link ru.ispras.retrascope.model.efsm.EfsmModel} model to GraphML file.
   *
   * @param fileName the output file name
   * @param efsm extended finite state machine model to be printed
   */
  public static void print(final String fileName, final EfsmModel efsm) {

    final EfsmGraphMlVisitor visitor = new EfsmGraphMlVisitor();
    visitor.initFile(fileName);

    final ModelWalker efsmWalker = new EfsmDfsWalker(visitor);
    efsmWalker.start(efsm);
  }

  @Override
  public String getDefaultFileName() {
    return "efsm-model.graphml";
  }
}
