/*
 * Copyright 2019 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package ru.ispras.retrascope.engine.gadd.transformer.hldd.formula;

import ru.ispras.fortress.expression.NodeVariable;

import java.util.LinkedList;
import java.util.List;

/**
 * This class represents a descriptor of a variable labeled by phases.
 *
 * @author <a href="mailto:lebedev@ispras.ru">Mikhail Lebedev</a>
 */
public final class PhasedDescriptor {

  private final List<Integer> previousPhases;
  private final NodeVariable original;
  private final boolean isGuard;

  /**
   * Constructs a descriptor for a variable.
   *
   * @param original       - the original variable.
   * @param previousPhases - the list of previous phases.
   */
  public PhasedDescriptor(final NodeVariable original, final List<Integer> previousPhases) {
    this.previousPhases = new LinkedList<>(previousPhases);
    this.original = original;
    this.isGuard = false;
  }

  /**
   * Constructs a descriptor for a guard.
   */
  public PhasedDescriptor() {
    this.previousPhases = new LinkedList<>();
    this.original = null;
    this.isGuard = true;
  }

  /**
   * Indicates that a descriptor belongs to a guard.
   *
   * @return {@code true} if a descriptor belongs to a guard, {@code false} otherwise.
   */
  public boolean isGuard() {
    return isGuard;
  }

  /**
   * Returns the list of previous phases.
   *
   * @return The list of previous phases.
   */
  public List<Integer> getPreviousPhases() {
    return previousPhases;
  }

  /**
   * Returns the original variable.
   *
   * @return The original variable.
   */
  public NodeVariable getOriginal() {
    return original;
  }
}
