/*
 * Copyright 2018 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package ru.ispras.retrascope.engine.hldd.printer.smv.basis;

import org.apache.commons.cli.ParseException;
import ru.ispras.retrascope.basis.Arguments;
import ru.ispras.retrascope.basis.Entity;
import ru.ispras.retrascope.basis.EntityType;
import ru.ispras.retrascope.basis.FileNames;
import ru.ispras.retrascope.basis.exception.RetrascopeException;
import ru.ispras.retrascope.model.hldd.HlddModel;
import ru.ispras.retrascope.model.hldd.HlddModelUtils;
import ru.ispras.retrascope.model.hldd.HlddVisitor;
import ru.ispras.retrascope.model.hldd.HlddWalker;

import java.util.Map;

/**
 * This is the base class for HLDD-SMV printers.
 *
 * @author <a href="mailto:lebedev@ispras.ru">Mikhail Lebedev</a>
 */
public abstract class HlddSmvPrinterBase extends SmvPrinterEngine {

  protected HlddSmvPrinterBase(final String engineId) {
    super(engineId);
    addInputType(HlddModel.TYPE);
  }

  @Override
  public Entity start(final Map<EntityType, Entity> inputs) {
    final FileNames result = new FileNames(OUT_TYPE.getId());
    try {
      final HlddModel model = (HlddModel) inputs.get(HlddModel.TYPE);
      if (!HlddModelUtils.containsEmptyModules(model)) {
        final Arguments arguments =
            parseCommandLine(((Arguments) inputs.get(Arguments.TYPE)).getCommandLine());
        final String fileName = getSmvFileName(arguments);
        result.add(fileName);
        walkModel(fileName, model, inputs, arguments);
      }
    } catch (final ParseException e) {
      throw new RetrascopeException(e);
    }
    return result;
  }

  protected void walkModel(final String fileName, final HlddModel model,
      final Map<EntityType, Entity> inputs, final Arguments arguments) {
    final HlddVisitor visitor = getVisitor(fileName);
    configureVisitor(inputs, arguments, visitor);
    final HlddWalker walker = new HlddWalker(visitor);
    walker.start(model);
  }

  /*
   * Returns the specific HLDD visitor.
   */
  protected abstract HlddVisitor getVisitor(final String fileName);

  /*
   * Configures a visitor using parameters from the command line.
   */
  protected abstract void configureVisitor(final Map<EntityType, Entity> inputs,
      final Arguments args, final HlddVisitor visitor);
}
