/*
 * Copyright 2018 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package ru.ispras.retrascope.engine.hldd.printer.smv.basis;

import java.util.HashMap;
import java.util.Map;

/**
 * This enum represents different methods for model checking.
 *
 * @author <a href="mailto:lebedev@ispras.ru">Mikhail Lebedev</a>
 */
public enum ModelCheckingMethod {
  LTL_BMC("ltl-bmc"),
  LTL_MSAT("ltl-msat"),
  PSL_BMC("psl-bmc"),
  CTL_BDD("ctl-bdd"),
  PSL_BDD("psl-bdd");

  private static final Map<String, ModelCheckingMethod> VALUE_MAP = new HashMap<>();
  private String value;

  static {
    for (ModelCheckingMethod method : ModelCheckingMethod.values()) {
      VALUE_MAP.put(method.getStringValue(), method);
    }
  }

  ModelCheckingMethod(final String string) {
    value = string;
  }

  /**
   * Returns the associated string value of the enum element.
   *
   * @return The string value.
   */
  public String getStringValue() {
    return value;
  }

  /**
   * Returns the enum value based on the specified string value.
   *
   * @param string - the specified string.
   * @return The enum value.
   */
  public static ModelCheckingMethod getEnumValue(final String string) {
    return VALUE_MAP.get(string);
  }

  /**
   * Returns the associated string values as an array.
   *
   * @return The array of strings.
   */
  public static String[] getStringValues() {
    return VALUE_MAP.keySet().toArray(new String[VALUE_MAP.size()]);
  }
}
