/*
 * Copyright 2018 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package ru.ispras.retrascope.engine.hldd.printer.smv.usedef;

import ru.ispras.retrascope.result.usedef.UseDefContainer;

/**
 * @author <a href="mailto:lebedev@ispras.ru">Mikhail Lebedev</a>
 */
final class HlddUseDefUtils {

  private HlddUseDefUtils() {

  }

  /**
   * Shows whether the specified defines and uses belong to the same process.
   *
   * @param useDefContainer - the specified use/def container.
   * @return {@code true} if defines and uses belong to the same process, {@code false} otherwise.
   */
  public static boolean useDefSameProcess(final UseDefContainer useDefContainer) {
    if (useDefContainer.getUses().size() == 1 && useDefContainer.getDefines().size() == 1) {
      final String useProcess = useDefContainer.getUses().keySet().iterator().next();
      final String defProcess = useDefContainer.getDefines().keySet().iterator().next();
      return useProcess.compareTo(defProcess) == 0;
    }
    return false;
  }
}
