/*
 * Copyright 2018 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package ru.ispras.retrascope.engine.hldd.printer.smv.utils;

/**
 * This enum represents the SMV language reserved words.
 *
 * @author <a href="mailto:lebedev@ispras.ru">Mikhail Lebedev</a>
 */
public enum SmvReservedWords {

  /**
   * Announces module declaration.
   */
  MODULE("MODULE"),

  /**
   * Variable declaration section header.
   */
  VARIABLE("VAR"),

  /**
   * Input variable declaration section header.
   */
  INPUT_VARIABLE("IVAR"),

  /**
   * Assignment section header.
   */
  ASSIGNMENT("ASSIGN"),

  /**
   * Definition section header.
   */
  DEFINE("DEFINE"),

  /**
   * CTL specification header.
   */
  CTLSPEC("CTLSPEC"),

  /**
   * LTL specification header.
   */
  LTLSPEC("LTLSPEC"),

  /**
   * PSL specification header.
   */
  PSLSPEC("PSLSPEC"),

  /**
   * Invariant header.
   */
  INVAR("INVAR"),

  /**
   * Transition header.
   */
  TRANSITION("TRANS"),

  /**
   * Initialization assignment keyword.
   */
  INIT("init"),

  /**
   * Next assignment keyword.
   */
  NEXT("next"),

  /**
   * Case begin keyword.
   */
  CASE_BEGIN("case"),

  /**
   * Case end keyword.
   */
  CASE_END("esac"),

  /**
   * To integer conversion keyword.
   */
  TOINT("toint"),

  /**
   * To boolean conversion keyword.
   */
  TOBOOL("bool"),

  /**
   * To bit conversion keyword.
   */
  TOBIT("word1"),

  /**
   * Integer to bit-vector conversion keyword.
   */
  INT2BV("uwconst"),

  /**
   * True value.
   */
  TRUE("TRUE"),

  /**
   * False value.
   */
  FALSE("FALSE"),

  /**
   * Signed attribute.
   */
  SIGNED("signed"),

  /**
   * Unsigned attribute.
   */
  UNSIGNED("unsigned"),

  /**
   * Boolean data type.
   */
  BOOLEAN("boolean"),

  /**
   * Bit vector data type.
   */
  BIT_VECTOR("word"),

  /**
   * Integer data type.
   */
  INTEGER("integer"),

  /**
   * Array data type.
   */
  ARRAY("array"),

  /**
   * Reserved main module name.
   */
  MAIN_MODULE("main"),

  /**
   * Invariant specification header.
   */
  INVARSPEC("INVARSPEC");

  private String keyword;

  SmvReservedWords(final String keyword) {
    this.keyword = keyword;
  }

  /**
   * Specific SMV operations.
   */
  public enum Operation {
    NEXT;
  }

  @Override
  public String toString() {
    return keyword;
  }

  /**
   * All SMV language reserved words.
   */
  public static final String[] RESERVED_WORDS =
      {"A", "ABF", "ABG", "AF", "AG", ARRAY.toString(), ASSIGNMENT.toString(), "AX",
          TOBOOL.toString(), BOOLEAN.toString(), "BU", CASE_BEGIN.toString(), "COMPASSION",
          "COMPID", "COMPUTE", "COMPWFF", "CONSTANTS", "CONSTRAINT", "count", CTLSPEC.toString(),
          "CTLWFF", DEFINE.toString(), "E", "EBF", "EBG", "EF", "EG", CASE_END.toString(), "EX",
          "extend", "F", "FAIRNESS", FALSE.toString(), "floor", "FROZENVAR", "FUN", "G", "H", "IN",
          "in", "INIT", INIT.toString(), "Integer", INTEGER.toString(), INVAR.toString(),
          INVARSPEC.toString(), "ISA", "ITYPE", INPUT_VARIABLE.toString(), "JUSTICE",
          LTLSPEC.toString(), "LTLWFF", MAIN_MODULE.toString(), "MAX", "max", "MDEFINE", "MIN",
          "MIRROR", "mod", MODULE.toString(), "NAME", NEXT.toString(), "NEXTWFF", "O", "of", "PRED",
          "PREDICATES", PSLSPEC.toString(), "READ", "Real", "real", "resize", "S", "self",
          SIGNED.toString(), "SIMPWFF", "sizeof", "SPEC", "swconst", "T", TOINT.toString(),
          TRANSITION.toString(), TRUE.toString(), "U", "union", UNSIGNED.toString(),
          INT2BV.toString(), "V", VARIABLE.toString(), "Word", BIT_VECTOR.toString(),
          TOBIT.toString(), "WRITE", "X", "xnor", "xor", "Y", "Z"};
}
