/*
 * Copyright 2014-2018 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package ru.ispras.retrascope.engine.test.printer.testbench.vhdl;

import ru.ispras.fortress.data.DataType;
import ru.ispras.retrascope.basis.Entity;
import ru.ispras.retrascope.basis.EntityType;
import ru.ispras.retrascope.engine.test.printer.testbench.TestbenchPrinterEngine;
import ru.ispras.retrascope.result.testbench.VhdlTestbenchModel;

import java.util.HashMap;
import java.util.Map;
import java.util.TreeMap;

/**
 * The engine that makes VHDL testbenches from tests.
 *
 * @author <a href="mailto:i.melnichenko@deltasolutions.ru">Igor Melnichenko</a>
 * @author <a href="mailto:smolov@ispras.ru">Sergey Smolov</a>
 */
public final class TestVhdlTestbenchPrinter extends TestbenchPrinterEngine {

  /**
   * The engine ID of all {@code TestVhdlTestbenchPrinter} instances.
   */
  private static final String ENGINE_ID = "test-vhdl-testbench-printer";

  private static final Map<DataType, String> DATA_TYPES_MAP = new HashMap<>();

  private static final Map<String, String> CONVERT_FUNC_MAP = new TreeMap<>(
      String.CASE_INSENSITIVE_ORDER);

  /**
   * Creates a new {@code TestVhdlTestbenchPrinter} instance.
   */
  public TestVhdlTestbenchPrinter() {
    super(ENGINE_ID, VhdlTestbenchModel.TYPE);
  }

  static {
    DATA_TYPES_MAP.put(DataType.bitVector(1), "std_logic");
    DATA_TYPES_MAP.put(DataType.INTEGER, "integer");
    DATA_TYPES_MAP.put(DataType.BOOLEAN, "bit");
    DATA_TYPES_MAP.put(DataType.REAL, "real");

    CONVERT_FUNC_MAP.put("std_logic", "string_to_std_logic");
    CONVERT_FUNC_MAP.put("std_logic_vector", "string_to_std_logic_vector");
    CONVERT_FUNC_MAP.put("bit", "string_to_bit");
    CONVERT_FUNC_MAP.put("bit_vector", "string_to_bit_vector");
    CONVERT_FUNC_MAP.put("integer", "string_to_integer");
    CONVERT_FUNC_MAP.put("real", "string_to_real");
  }

  @Override
  public String getTestbenchExt() {
    return "vhd";
  }

  @Override
  protected String getTemplateDirName() {
    return "stg/vhdl/";
  }

  @Override
  protected String getTemplateName() {
    return "VhdlTestbench.stg";
  }

  @Override
  protected String getTstParserName() {
    return "VhdlTstParser.stg";
  }

  @Override
  protected String getStEntityName() {
    return "vhdl_testbench";
  }

  @Override
  public String getInputPortKind() {
    return "in";
  }

  @Override
  public String getOutputPortKind() {
    return "out";
  }

  @Override
  public String getBitVectorPrefix() {
    return "std_logic_vector (";
  }

  @Override
  public String getBitVectorDelim() {
    return " to ";
  }

  @Override
  public String getBitVectorSuffix() {
    return ")";
  }

  @Override
  public String getInputDeclarationFmt() {
    return "signal";
  }

  @Override
  public String getOutputDeclarationFmt() {
    return "signal";
  }

  @Override
  public Map<DataType, String> getDataTypeMap() {
    return DATA_TYPES_MAP;
  }

  @Override
  public Map<String, String> getConvertFunctionMap() {
    return CONVERT_FUNC_MAP;
  }

  @Override
  public Entity start(Map<EntityType, Entity> inputs) {

    return new VhdlTestbenchModel(makeTestbenches(inputs));
  }
}
