/*
 * Copyright 2015-2017 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package ru.ispras.retrascope.model.efsm;

import ru.ispras.retrascope.model.basis.walker.ModelEmptyVisitor;

/**
 * An empty visitor for EfsmModel.
 * <p>
 * Does nothing on every sub-component of EfsmModel.
 * </p>
 * @see EfsmVisitor
 * @see EfsmModel
 *
 * @author <a href="mailto:smolov@ispras.ru">Sergey Smolov</a>
 */
public class EfsmEmptyVisitor extends ModelEmptyVisitor implements EfsmVisitor {

  @Override
  public void onStateBegin(final EfsmState state) {
    /* Empty. */
  }

  @Override
  public void onStateEnd(final EfsmState state) {
    /* Empty. */
  }

  @Override
  public void onTransitionBegin(final EfsmTransition transition) {
    /* Empty. */
  }

  @Override
  public void onTransitionEnd(final EfsmTransition transition) {
    /* Empty. */
  }

  @Override
  public void setStatus(final Status status) {
    /* Empty. */
  }
}
