/*
 * Copyright 2015-2017 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package ru.ispras.retrascope.parser.basis.backend;

import ru.ispras.fortress.expression.NodeValue;
import ru.ispras.fortress.expression.Nodes;
import ru.ispras.retrascope.basis.exception.RetrascopeException;
import ru.ispras.retrascope.model.basis.Statement;
import ru.ispras.retrascope.model.basis.StatementType;
import ru.ispras.retrascope.model.basis.StatementUtils;
import ru.ispras.retrascope.model.cfg.CfgAssertStatement;
import ru.ispras.retrascope.model.cfg.CfgCaseStatement;
import ru.ispras.retrascope.model.cfg.CfgEmptyVisitor;
import ru.ispras.retrascope.model.cfg.CfgProcess;
import ru.ispras.retrascope.model.cfg.CfgSwitchStatement;
import ru.ispras.retrascope.util.LogMessage;

import java.util.LinkedList;
import java.util.List;

/**
 * Backend for the HDL parser that substitutes assert nodes by the equivalent statements.
 * @author <a href="mailto:smolov@ispras.ru">Sergey Smolov</a>
 */
public class CfgAssertBackend extends CfgEmptyVisitor {

  /**
   * The backend name.
   */
  public static final String NAME = "assert";

  @Override
  public void onAssertBegin(final CfgAssertStatement statement) {

    final CfgProcess process =
        (CfgProcess) StatementUtils.nearStatement(statement, StatementType.PROCESS);

    if (process == null) {
      throw RetrascopeException.exception(LogMessage.CANT_FIND, "nearest process.");
    }

    final Statement parent = statement.getParent();
    parent.remove(statement);

    /* Collect all statements of nearest variable container. */
    final List<Statement> containerStmts = new LinkedList<>();

    for (int i = 0; i < process.getStatementNum(); i++) {
      containerStmts.add(process.getStatement(i));
    }

    process.removeAll();

    /* Substitute assertion by top-level switch node that checks that condition is true. */
    final CfgSwitchStatement switchStatement = new CfgSwitchStatement(statement.getCondition());
    final CfgCaseStatement trueCase = new CfgCaseStatement((NodeValue) Nodes.TRUE);
    switchStatement.add(trueCase);
    trueCase.addAll(containerStmts);
    process.add(switchStatement);
  }
}
