/*
 * Copyright 2014-2016 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package ru.ispras.retrascope.result.test.jaxb;

import org.apache.commons.collections15.map.HashedMap;

import ru.ispras.fortress.expression.NodeValue;
import ru.ispras.retrascope.result.test.jaxb.JaxbVariableMapXmlAdapter.JaxbVariableList;
import ru.ispras.retrascope.result.test.jaxb.JaxbVariableMapXmlAdapter.JaxbVariableList.JaxbVariable;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import javax.xml.bind.annotation.XmlAttribute;
import javax.xml.bind.annotation.XmlElement;
import javax.xml.bind.annotation.adapters.XmlAdapter;

/**
 * The adapter class for JAXB marshalling/unmarshalling of variables.
 *
 * @author <a href="mailto:i.melnichenko@deltasolutions.ru">Igor Melnichenko</a>
 */
public class JaxbVariableMapXmlAdapter
    extends XmlAdapter<JaxbVariableList, Map<String, NodeValue>> {

  @Override
  public JaxbVariableList marshal(Map<String, NodeValue> map) throws Exception {
    JaxbVariableList jaxbVariableList = new JaxbVariableList();

    for (final Map.Entry<String, NodeValue> entry : map.entrySet()) {
      JaxbVariable jaxbVariable = new JaxbVariable();
      jaxbVariable.name = entry.getKey();
      jaxbVariable.data = entry.getValue();
      jaxbVariableList.jaxbVariables.add(jaxbVariable);
    }

    return jaxbVariableList;
  }

  @Override
  public Map<String, NodeValue> unmarshal(JaxbVariableList list) throws Exception {
    Map<String, NodeValue> variables = new HashedMap<>(list.jaxbVariables.size());

    for (JaxbVariable jaxbVariable: list.jaxbVariables) {
      variables.put(jaxbVariable.name, jaxbVariable.data);
    }

    return variables;
  }

  /**
   * The class for mediation during JAXB marshalling/unmarshalling of variables.
   *
   * @author <a href="mailto:i.melnichenko@deltasolutions.ru">Igor Melnichenko</a>
   */
  public static class JaxbVariableList {
    @XmlElement(name = "variable")
    public final List<JaxbVariable> jaxbVariables = new ArrayList<>();

    /**
     * The class for mediation during JAXB marshalling/unmarshalling of variables.
     *
     * @author <a href="mailto:i.melnichenko@deltasolutions.ru">Igor Melnichenko</a>
     */
    public static class JaxbVariable {
      @XmlAttribute
      public String name;
      @XmlElement
      public NodeValue data;
    }
  }
}
